@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -79,6 +79,7 @@ pop_number_of_operands(struct arithmeticparams *p, int op, char *token_string,
   /* See if this operator needs any parameters. If so, pop them. */
   switch(op)
     {
+    case GAL_ARITHMETIC_OP_STITCH:
     case GAL_ARITHMETIC_OP_QUANTILE:
       numparams=1;
       break;
@@ -104,7 +105,7 @@ pop_number_of_operands(struct arithmeticparams *p, int op, char *token_string,
       gal_list_data_add(params, tmp);
 
       /* A small sanity check (none of the parameters for sigma-clipping,
-         or quantile estimation can be negative. */
+         or quantile estimation can be negative). */
       if( ((float *)(tmp->array))[0]<=0.0 )
         error(EXIT_FAILURE, 0, "the %s popped operand of the '%s' "
               "operator cannot be negative", cstring, token_string);
@@ -113,7 +114,7 @@ pop_number_of_operands(struct arithmeticparams *p, int op, char *token_string,
       cstring=c?"third":"second";
     }
 
-  /* Check if its a number. */
+  /* Check if it is a number. */
   numpop=operands_pop(p, token_string);
   if(numpop->size>1)
     error(EXIT_FAILURE, 0, "the %s popped operand of the '%s' "
@@ -189,8 +190,6 @@ struct arithmetic_filter_p
   float      sclip_param;       /* Termination critera in sigma-cliping. */
   gal_data_t      *input;       /* Input dataset.                        */
   gal_data_t        *out;       /* Output dataset.                       */
-
-  int           hasblank;       /* If the dataset has blank values.      */
 };
 
 
@@ -385,18 +384,16 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
      pop the necessary number of operands. */
   nparams = ndim + (issigclip ? 2 : 0 );
   for(i=0;i<nparams;++i)
-    gal_list_data_add(&params_list, operands_pop(p, token));
-
-
-  /* Make sure the parameters only have single values. */
-  i=0;
-  for(tmp=params_list; tmp!=NULL; tmp=tmp->next)
     {
-      ++i;
-      if(tmp->size!=1)
+      /* Add this to the list of parameters. */
+      gal_list_data_add(&params_list, operands_pop(p, token));
+
+      /* Make sure it only has a single element. */
+      if(params_list->size!=1)
         error(EXIT_FAILURE, 0, "the parameters given to the filtering "
-              "operators can only be numbers. Value number %zu has %zu "
-              "elements, so its an array", i, tmp->size);
+              "operators can only be numbers. Operand number %zu after "
+              "the main input has %zu elements, so its an array", i,
+              params_list->size);
     }
 
 
@@ -417,10 +414,23 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
       gal_data_free(tmp);
     }
 
+  /* If the input only has one element, it is most probably and error (the
+     user has confused the parameters with the input dataset). So report a
+     warning filtering makes no sense, so don't waste time, just add the
+     input onto the stack. */
+  if(afp.input->size==1)
+    {
+      /* Inform the user that this is suspicious. */
+      if(p->cp.quiet==0)
+        error(EXIT_SUCCESS, 0, "WARNING: the first popped operand to the "
+              "filtering operators has a single element! This is most "
+              "probably a typo in the order of operands! Recall that the "
+              "filtering operators need the main input image/cube as the "
+              "first popped operand");
 
-  /* If the input only has one element, filtering makes no sense, so don't
-     waste time, just add the input onto the stack. */
-  if(afp.input->size==1) afp.out=afp.input;
+      /* Set the input as the output. */
+      afp.out=afp.input;
+    }
   else
     {
       /* Allocate an array for the size of the filter and fill it in. The
@@ -452,8 +462,8 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
           /* If the width is larger than the input's size, change the width
              to the input's size. */
           if( fsize[i] > afp.input->dsize[i] )
-            error(EXIT_FAILURE, 0, "%s: the filter size along dimension %zu "
-                  "(%zu) is greater than the input's length in that "
+            error(EXIT_FAILURE, 0, "%s: the filter size along dimension "
+                  "%zu (%zu) is greater than the input's length in that "
                   "dimension (%zu)", __func__, i, fsize[i],
                   afp.input->dsize[i]);
 
@@ -480,9 +490,6 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
       printf("hpfsize: %zu, %zu\n", hpfsize[0], hpfsize[1]);
       */
 
-      /* See if the input has blank pixels. */
-      afp.hasblank=gal_blank_present(afp.input, 1);
-
 
       /* Set the type of the output dataset. */
       switch(operator)
@@ -498,8 +505,8 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
           break;
 
         default:
-          error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s to fix "
-                "the problem. The 'operator' code %d is not recognized",
+          error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s to "
+                "fix the problem. The 'operator' code %d is not recognized",
                 PACKAGE_BUGREPORT, __func__, operator);
         }
 
@@ -522,10 +529,10 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
   /* Add the output to the top of the stack. */
   operands_add(p, NULL, afp.out);
 
-  /* Clean up and add the output on top of the stack */
+  /* Clean up and add the output on top of the stack. */
   gal_data_free(zero);
-  gal_data_free(afp.input);
   gal_list_data_free(params_list);
+  if(afp.input!=afp.out) gal_data_free(afp.input); /* Single-element. */
 }
 
 
@@ -577,10 +584,13 @@ arithmetic_binary_sanity_checks(gal_data_t *in, gal_data_t *conn,
 
   /* Make sure the array has an unsigned 8-bit type. */
   if(in->type!=GAL_TYPE_UINT8)
-    error(EXIT_FAILURE, 0, "the second popped operand of '%s' doesn't "
-          "have an 8-bit unsigned integer type. It must be a binary "
-          "dataset (only being equal to zero is checked). You can use "
-          "the 'uint8' operator for type conversion", operator);
+    error(EXIT_FAILURE, 0, "the second popped operand of '%s' has a type "
+          "of %s. However, it must be a binary dataset (only being equal "
+          "to zero is checked). You can use the 'uint8' operator for type "
+          "conversion, alternatively, if all your values are positive "
+          "and floating point, you can use '0 gt', if you want non-blank "
+          "values you can use 'isblank not' and many other operators that "
+          "produce a binary output", operator, gal_type_name(in->type, 1));
 
   /* Clean up and return the integer value of 'conn'. */
   gal_data_free(conn);
@@ -622,6 +632,27 @@ arithmetic_erode_dilate(struct arithmeticparams *p, char *token, int op)
 
 
 
+static void
+arithmetic_number_neighbors(struct arithmeticparams *p, char *token, int op)
+{
+  int conn_int;
+
+  /* Pop the two necessary operands. */
+  gal_data_t *conn = operands_pop(p, token);
+  gal_data_t *in   = operands_pop(p, token);
+
+  /* Do the sanity checks and do the job. */
+  conn_int=arithmetic_binary_sanity_checks(in, conn, token);
+  in=gal_binary_number_neighbors(in, conn_int, 1);
+
+  /* Push the result onto the stack. */
+  operands_add(p, NULL, in);
+}
+
+
+
+
+
 static void
 arithmetic_connected_components(struct arithmeticparams *p, char *token)
 {
@@ -661,7 +692,7 @@ arithmetic_fill_holes(struct arithmeticparams *p, char *token)
   /* Basic sanity checks. */
   conn_int=arithmetic_binary_sanity_checks(in, conn, token);
 
-  /* Fill the holes */
+  /* Fill the holes. */
   gal_binary_holes_fill(in, conn_int, -1);
 
   /* Push the result onto the stack. */
@@ -827,6 +858,7 @@ arithmetic_interpolate_region(struct arithmeticparams *p,
   */
 
   /* Clean up. */
+  free(dinc);
   gal_data_free(lab);
   gal_data_free(minmax);
   gal_data_free(connectivity);
@@ -845,7 +877,7 @@ arithmetic_interpolate(struct arithmeticparams *p, int operator, char *token)
   gal_data_t *interpolated;
   int num_int, interpop=GAL_ARITHMETIC_OP_INVALID;
 
-  /* First pop the number of nearby neighbors.*/
+  /* First pop the number of nearby neighbors. */
   gal_data_t *num = operands_pop(p, token);
 
   /* Then pop the actual dataset to interpolate. */
@@ -878,6 +910,9 @@ arithmetic_interpolate(struct arithmeticparams *p, int operator, char *token)
     case ARITHMETIC_OP_INTERPOLATE_MAXNGB:
       interpop=GAL_INTERPOLATE_NEIGHBORS_FUNC_MAX;
       break;
+    case ARITHMETIC_OP_INTERPOLATE_MEANNGB:
+      interpop=GAL_INTERPOLATE_NEIGHBORS_FUNC_MEAN;
+      break;
     case ARITHMETIC_OP_INTERPOLATE_MEDIANNGB:
       interpop=GAL_INTERPOLATE_NEIGHBORS_FUNC_MEDIAN;
       break;
@@ -902,32 +937,65 @@ arithmetic_interpolate(struct arithmeticparams *p, int operator, char *token)
 
 
 
+static void
+arithmetic_collapse_single_value(gal_data_t *input, char *counter,
+                                 char *opstr, char *description,
+                                 int checkint)
+{
+  if( input->ndim!=1 || input->size!=1)
+    error(EXIT_FAILURE, 0, "%s popped operand of 'collapse-%s*' operators "
+          "(%s) must be a single number (single-element, "
+          "one-dimensional dataset). But it has %zu dimension(s) and %zu "
+          "element(s).", counter, opstr, description, input->ndim, input->size);
+  if(checkint)
+    if(input->type==GAL_TYPE_FLOAT32 || input->type==GAL_TYPE_FLOAT64)
+      error(EXIT_FAILURE, 0, "%s popped operand of 'collapse-%s*' operators "
+            "(%s) must have an integer type, but it has a floating point "
+            "type ('%s')", counter, opstr, description,
+            gal_type_name(input->type,1));
+}
+
+
+
+
+
 static void
 arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
 {
   long dim;
-  gal_data_t *collapsed=NULL;
+  float p1=NAN, p2=NAN;
+  int qmm=p->cp.quietmmap;
+  gal_data_t *dimension=NULL, *input=NULL;
+  size_t nt=p->cp.numthreads, mms=p->cp.minmapsize;
+  gal_data_t *collapsed=NULL, *param1=NULL, *param2=NULL;
+
 
   /* First popped operand is the dimension. */
-  gal_data_t *dimension = operands_pop(p, token);
-
-  /* The second popped operand is the desired input dataset. */
-  gal_data_t *input = operands_pop(p, token);
+  dimension = operands_pop(p, token);
 
 
-  /* Small sanity check. */
-  if( dimension->ndim!=1 || dimension->size!=1)
-    error(EXIT_FAILURE, 0, "first popped operand of 'collapse-*' operators "
-          "(dimension to collapse) must be a single number (single-element, "
-          "one-dimensional dataset). But it has %zu dimension(s) and %zu "
-          "element(s).", dimension->ndim, dimension->size);
-  if(dimension->type==GAL_TYPE_FLOAT32 || dimension->type==GAL_TYPE_FLOAT64)
-    error(EXIT_FAILURE, 0, "first popped operand of 'collapse-*' operators "
-          "(dimension to collapse) must have an integer type, but it has "
-          "a floating point type ('%s')", gal_type_name(dimension->type,1));
+  /* If we are on any of the sigma-clipping operators, we need to pop two
+     more operands. */
+  if(    operator==ARITHMETIC_OP_COLLAPSE_SIGCLIP_STD
+      || operator==ARITHMETIC_OP_COLLAPSE_SIGCLIP_MEAN
+      || operator==ARITHMETIC_OP_COLLAPSE_SIGCLIP_MEDIAN
+      || operator==ARITHMETIC_OP_COLLAPSE_SIGCLIP_NUMBER )
+    {
+      param2=operands_pop(p, token); /* Termination criteria. */
+      param1=operands_pop(p, token); /* Multiple of sigma. */
+    }
+
+
+  /* Final popped operand is the desired input dataset. */
+  input = operands_pop(p, token);
+
+
+  /* Sanity checks. */
+  arithmetic_collapse_single_value(dimension, "first", "",
+                                   "dimension to collapse", 1);
   dimension=gal_data_copy_to_new_type_free(dimension, GAL_TYPE_LONG);
   dim=((long *)(dimension->array))[0];
-  if(dim<0 || dim==0)
+  if(dim<=0)
     error(EXIT_FAILURE, 0, "first popped operand of 'collapse-*' operators "
           "(dimension to collapse) must be positive (larger than zero), it "
           "is %ld", dim);
@@ -935,7 +1003,22 @@ arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
     error(EXIT_FAILURE, 0, "input dataset to '%s' has %zu dimension(s), "
           "but you have asked to collapse along dimension %ld", token,
           input->ndim, dim);
-
+  if(param1)
+    {
+      arithmetic_collapse_single_value(param1, "third", "sigclip-",
+                                       "sigma-clip's multiple of sigma", 0);
+      arithmetic_collapse_single_value(param2, "second", "sigclip-",
+                                       "sigma-clip's termination param", 0);
+      param1=gal_data_copy_to_new_type_free(param1, GAL_TYPE_FLOAT32);
+      param2=gal_data_copy_to_new_type_free(param2, GAL_TYPE_FLOAT32);
+      p1=((float *)(param1->array))[0];
+      p2=((float *)(param2->array))[0];
+      if(p1<=0 || p2<=0)
+        error(EXIT_FAILURE, 0, "third and second popped operands of "
+              "'collapse-sigclip-*' operators (sigma multiple and "
+              "termination criteria) must be positive (larger than "
+              "zero), but they are %g and %g", p1, p2);
+    }
 
   /* If a WCS structure has been read, we'll need to pass it to
      'gal_dimension_collapse', so it modifies it respectively. */
@@ -969,6 +1052,31 @@ arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
       collapsed=gal_dimension_collapse_minmax(input, input->ndim-dim, 1);
       break;
 
+    case ARITHMETIC_OP_COLLAPSE_MEDIAN:
+      collapsed=gal_dimension_collapse_median(input, input->ndim-dim,
+                                              nt, mms, qmm);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_SIGCLIP_STD:
+      collapsed=gal_dimension_collapse_sclip_std(input, input->ndim-dim,
+                                                 p1, p2, nt, mms, qmm);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_SIGCLIP_MEAN:
+      collapsed=gal_dimension_collapse_sclip_mean(input, input->ndim-dim,
+                                                  p1, p2, nt, mms, qmm);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_SIGCLIP_MEDIAN:
+      collapsed=gal_dimension_collapse_sclip_median(input, input->ndim-dim,
+                                                    p1, p2, nt, mms, qmm);
+      break;
+
+    case ARITHMETIC_OP_COLLAPSE_SIGCLIP_NUMBER:
+      collapsed=gal_dimension_collapse_sclip_number(input, input->ndim-dim,
+                                                    p1, p2, nt, mms, qmm);
+      break;
+
     default:
       error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix the "
             "problem. The operator code %d is not recognized", __func__,
@@ -1020,7 +1128,7 @@ arithmetic_tofile(struct arithmeticparams *p, char *token, int freeflag)
                            : OPERATOR_PREFIX_LENGTH_TOFILE     ];
 
   /* Make sure that if the file exists, it is deleted. */
-  gal_checkset_writable_remove(filename, p->cp.keep,
+  gal_checkset_writable_remove(filename, NULL, p->cp.keep,
                                p->cp.dontdelete);
 
   /* Save it to a file. */
@@ -1043,18 +1151,6 @@ arithmetic_tofile(struct arithmeticparams *p, char *token, int freeflag)
 
 
 
-void
-arithmetic_unique(struct arithmeticparams *p, char *token, int operator)
-{
-  /* Pass the popped operand to the statistics library. */
-  gal_data_t *input = gal_statistics_unique(operands_pop(p, token), 1);
-  operands_add(p, NULL, input);
-}
-
-
-
-
-
 void
 arithmetic_add_dimension(struct arithmeticparams *p, char *token,
                          int operator)
@@ -1232,15 +1328,18 @@ arithmetic_repeat(struct arithmeticparams *p, char *token, int operator)
 /*************      Reverse Polish algorithm       *************/
 /***************************************************************/
 static int
-arithmetic_set_operator(char *string, size_t *num_operands)
+arithmetic_set_operator(char *string, size_t *num_operands, int *inlib)
 {
   /* Use the library's main function for its own operators. */
   int op = gal_arithmetic_set_operator(string, num_operands);
 
+  /* Mark if this operator is in the library or not. */
+  *inlib = op==GAL_ARITHMETIC_OP_INVALID ? 0 : 1;
+
   /* If its not a library operator, check if its an internal operator. */
   if(op==GAL_ARITHMETIC_OP_INVALID)
     {
-      /* Non-library operators. */
+      /* See if its a custom operator to the Arithmetic program. */
       if      (!strcmp(string, "filter-mean"))
         { op=ARITHMETIC_OP_FILTER_MEAN;           *num_operands=0; }
       else if (!strcmp(string, "filter-median"))
@@ -1253,6 +1352,8 @@ arithmetic_set_operator(char *string, size_t *num_operands)
         { op=ARITHMETIC_OP_ERODE;                 *num_operands=0; }
       else if (!strcmp(string, "dilate"))
         { op=ARITHMETIC_OP_DILATE;                *num_operands=0; }
+      else if (!strcmp(string, "number-neighbors"))
+        { op=ARITHMETIC_OP_NUMBER_NEIGHBORS;      *num_operands=0; }
       else if (!strcmp(string, "connected-components"))
         { op=ARITHMETIC_OP_CONNECTED_COMPONENTS;  *num_operands=0; }
       else if (!strcmp(string, "fill-holes"))
@@ -1263,6 +1364,8 @@ arithmetic_set_operator(char *string, size_t *num_operands)
         { op=ARITHMETIC_OP_INTERPOLATE_MINNGB;    *num_operands=0; }
       else if (!strcmp(string, "interpolate-maxngb"))
         { op=ARITHMETIC_OP_INTERPOLATE_MAXNGB;    *num_operands=0; }
+      else if (!strcmp(string, "interpolate-meanngb"))
+        { op=ARITHMETIC_OP_INTERPOLATE_MEANNGB;   *num_operands=0; }
       else if (!strcmp(string, "interpolate-medianngb"))
         { op=ARITHMETIC_OP_INTERPOLATE_MEDIANNGB; *num_operands=0; }
       else if (!strcmp(string, "interpolate-minofregion"))
@@ -1278,9 +1381,17 @@ arithmetic_set_operator(char *string, size_t *num_operands)
       else if (!strcmp(string, "collapse-mean"))
         { op=ARITHMETIC_OP_COLLAPSE_MEAN;         *num_operands=0; }
       else if (!strcmp(string, "collapse-number"))
-        { op=ARITHMETIC_OP_COLLAPSE_NUMBER;       *num_operands=0; }
-      else if (!strcmp(string, "unique"))
-        { op=ARITHMETIC_OP_UNIQUE;                *num_operands=0; }
+        { op=ARITHMETIC_OP_COLLAPSE_MEDIAN;       *num_operands=0; }
+      else if (!strcmp(string, "collapse-median"))
+        { op=ARITHMETIC_OP_COLLAPSE_MEDIAN;       *num_operands=0; }
+      else if (!strcmp(string, "collapse-sigclip-std"))
+        { op=ARITHMETIC_OP_COLLAPSE_SIGCLIP_STD;  *num_operands=0; }
+      else if (!strcmp(string, "collapse-sigclip-mean"))
+        { op=ARITHMETIC_OP_COLLAPSE_SIGCLIP_MEAN; *num_operands=0; }
+      else if (!strcmp(string, "collapse-sigclip-median"))
+        { op=ARITHMETIC_OP_COLLAPSE_SIGCLIP_MEDIAN;*num_operands=0;}
+      else if (!strcmp(string, "collapse-sigclip-number"))
+        { op=ARITHMETIC_OP_COLLAPSE_SIGCLIP_NUMBER;*num_operands=0;}
       else if (!strcmp(string, "add-dimension-slow"))
         { op=ARITHMETIC_OP_ADD_DIMENSION_SLOW;    *num_operands=0; }
       else if (!strcmp(string, "add-dimension-fast"))
@@ -1301,21 +1412,49 @@ arithmetic_set_operator(char *string, size_t *num_operands)
 
 
 
+static gal_data_t *
+arithmetic_prepare_meta(gal_data_t *d1, gal_data_t *d2, gal_data_t *d3)
+{
+  size_t i;
+  gal_data_t *out;
+
+  /* A small sanity check (these operators only need a single
+     operator). */
+  if(d2 || d3)
+    error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at '%s' to "
+          "find and fix the problem. Meta operators should only "
+          "take a single operand, but for some reason, this "
+          "hasn't occurred here", __func__, PACKAGE_BUGREPORT);
+
+  /* Allocate the output dataset. */
+  out=gal_data_alloc_empty(d1->ndim, d1->minmapsize, d1->quietmmap);
+
+  /* Fill the output dataset and return. */
+  for(i=0;i<d1->ndim;++i) out->dsize[i]=d1->dsize[i];
+  out->size=d1->size;
+  return out;
+}
+
+
+
+
+
 static void
 arithmetic_operator_run(struct arithmeticparams *p, int operator,
-                        char *operator_string, size_t num_operands)
+                        char *operator_string, size_t num_operands,
+                        int inlib)
 {
   size_t i;
   unsigned int numop;
   int flags = GAL_ARITHMETIC_FLAGS_BASIC;
-  gal_data_t *d1=NULL, *d2=NULL, *d3=NULL;
+  gal_data_t *d1=NULL, *d2=NULL, *d3=NULL, *d4=NULL;
 
   /* Set the operating-mode flags if necessary. */
   if(p->cp.quiet) flags |= GAL_ARITHMETIC_FLAG_QUIET;
   if(p->envseed)  flags |= GAL_ARITHMETIC_FLAG_ENVSEED;
 
-  /* When 'num_operands!=0', the operator is in the library. */
-  if(num_operands)
+  /* If this operator is in the library, we should pop everything here.  */
+  if(inlib)
     {
       /* Pop the necessary number of operators. Note that the
          operators are poped from a linked list (which is
@@ -1324,6 +1463,9 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
          last (right most, in in-fix notation) input operand.*/
       switch(num_operands)
         {
+        case 0:
+          break;
+
         case 1:
           d1=operands_pop(p, operator_string);
           break;
@@ -1339,12 +1481,19 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
           d1=operands_pop(p, operator_string);
           break;
 
+        case 4:
+          d4=operands_pop(p, operator_string);
+          d3=operands_pop(p, operator_string);
+          d2=operands_pop(p, operator_string);
+          d1=operands_pop(p, operator_string);
+          break;
+
         case -1:
           /* This case is when the number of operands is itself an
-             operand. So except for sigma-clipping (that has other
-             parameters), the first popped operand must be an
-             integer number, we will use that to construct a linked
-             list of any number of operands within the single 'd1'
+             operand. So except for operators that have high-level
+             parameters (like sigma-clipping), the first popped operand
+             must be an integer number, we will use that to construct a
+             linked list of any number of operands within the single 'd1'
              pointer. */
           numop=pop_number_of_operands(p, operator, operator_string, &d2);
           for(i=0;i<numop;++i)
@@ -1352,10 +1501,23 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
           break;
 
         default:
-          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
-                "the problem. '%zu' is not recognized as an operand "
-                "counter (with '%s')", __func__, PACKAGE_BUGREPORT,
-                num_operands, operator_string);
+          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s "
+                "to fix the problem. '%zu' is not recognized as an "
+                "operand counter (with '%s')", __func__,
+                PACKAGE_BUGREPORT, num_operands, operator_string);
+        }
+
+      /* Meta operators (where only the information about the pixels is
+         relevant, not the pixel values themselves, like the 'index'
+         operator). For these (that only accept one operand), we should add
+         the first popped operand back on the stack, then  */
+      switch(operator)
+        {
+        case GAL_ARITHMETIC_OP_INDEX:
+        case GAL_ARITHMETIC_OP_COUNTER:
+          operands_add(p, NULL, d1);
+          d1=arithmetic_prepare_meta(d1, d2, d3);
+          break;
         }
 
       /* Run the arithmetic operation. Note that 'gal_arithmetic'
@@ -1364,11 +1526,11 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
          when the operator doesn't need three operands, the extra
          arguments will be ignored. */
       operands_add(p, NULL, gal_arithmetic(operator, p->cp.numthreads,
-                                           flags, d1, d2, d3));
+                                           flags, d1, d2, d3, d4));
     }
 
-  /* No need to call the arithmetic library, call the proper
-     wrappers directly. */
+  /* No need to call the arithmetic library, call the proper wrappers
+     directly. */
   else
     {
       switch(operator)
@@ -1385,6 +1547,10 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
           arithmetic_erode_dilate(p, operator_string, operator);
           break;
 
+        case ARITHMETIC_OP_NUMBER_NEIGHBORS:
+          arithmetic_number_neighbors(p, operator_string, operator);
+          break;
+
         case ARITHMETIC_OP_CONNECTED_COMPONENTS:
           arithmetic_connected_components(p, operator_string);
           break;
@@ -1399,6 +1565,7 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
 
         case ARITHMETIC_OP_INTERPOLATE_MINNGB:
         case ARITHMETIC_OP_INTERPOLATE_MAXNGB:
+        case ARITHMETIC_OP_INTERPOLATE_MEANNGB:
         case ARITHMETIC_OP_INTERPOLATE_MEDIANNGB:
           arithmetic_interpolate(p, operator, operator_string);
           break;
@@ -1412,14 +1579,15 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
         case ARITHMETIC_OP_COLLAPSE_MIN:
         case ARITHMETIC_OP_COLLAPSE_MAX:
         case ARITHMETIC_OP_COLLAPSE_MEAN:
+        case ARITHMETIC_OP_COLLAPSE_MEDIAN:
         case ARITHMETIC_OP_COLLAPSE_NUMBER:
+        case ARITHMETIC_OP_COLLAPSE_SIGCLIP_STD:
+        case ARITHMETIC_OP_COLLAPSE_SIGCLIP_MEAN:
+        case ARITHMETIC_OP_COLLAPSE_SIGCLIP_MEDIAN:
+        case ARITHMETIC_OP_COLLAPSE_SIGCLIP_NUMBER:
           arithmetic_collapse(p, operator_string, operator);
           break;
 
-        case ARITHMETIC_OP_UNIQUE:
-          arithmetic_unique(p, operator_string, operator);
-          break;
-
         case ARITHMETIC_OP_ADD_DIMENSION_SLOW:
         case ARITHMETIC_OP_ADD_DIMENSION_FAST:
           arithmetic_add_dimension(p, operator_string, operator);
@@ -1464,6 +1632,66 @@ arithmetic_set_name_used_later(void *in, char *name)
 
 
 
+static void
+arithmetic_final_read_file(struct arithmeticparams *p,
+                           struct operand *operand)
+{
+  gal_data_t *out=NULL;
+  char *hdu, *filename;
+
+  /* Read the desired image and report it if necessary. */
+  hdu=operand->hdu;
+  filename=operand->filename;
+  if( gal_fits_file_recognized(filename) )
+    {
+      /* Read the data, note that the WCS has already been set. */
+      out=gal_array_read_one_ch(filename, hdu, NULL,
+                                p->cp.minmapsize,
+                                p->cp.quietmmap);
+      out->ndim=gal_dimension_remove_extra(out->ndim, out->dsize,
+                                            NULL);
+      if(!p->cp.quiet) printf(" - %s (hdu %s) is read.\n", filename, hdu);
+    }
+  else
+    error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s to fix "
+          "the problem. While 'operands->data' is NULL, the filename "
+          "('%s') is not recognized as a FITS file", __func__,
+          PACKAGE_BUGREPORT, filename);
+
+  /* Keep the read dataset (note that we don't need to free 'filename' or
+     'hdu' since their pointers are simply copied into the operand, they
+     are not copied.*/
+  operand->data=out;
+}
+
+
+
+
+
+/* Extract all the datasets of the remaining operands. */
+static gal_data_t *
+arithmetic_final_data(struct arithmeticparams *p)
+{
+  struct operand *op;
+  gal_data_t *out=NULL;
+
+  /* Go over the operands, extract their dataset and add it to the list. */
+  for(op=p->operands; op!=NULL; op=op->next)
+    {
+      gal_list_data_add(&out, op->data);
+      out->wcs=gal_wcs_copy(p->refdata.wcs);
+    }
+
+  /* Reverse the list so it goes in the same order as the operands, then
+     return it. */
+  gal_list_data_reverse(&out);
+  return out;
+}
+
+
+
+
+
 /* This function implements the reverse polish algorithm as explained
    in the Wikipedia page.
 
@@ -1472,11 +1700,13 @@ arithmetic_set_name_used_later(void *in, char *name)
 void
 reversepolish(struct arithmeticparams *p)
 {
-  gal_data_t *data;
+  char *printnum;
+  struct operand *otmp;
   size_t num_operands=0;
   gal_list_str_t *token;
-  char *hdu, *filename, *printnum;
-  int operator=GAL_ARITHMETIC_OP_INVALID;
+  gal_data_t *tmp, *data, *col;
+  struct gal_options_common_params *cp=&p->cp;
+  int inlib, operator=GAL_ARITHMETIC_OP_INVALID;
 
   /* Prepare the processing: */
   p->popcounter=0;
@@ -1505,6 +1735,11 @@ reversepolish(struct arithmeticparams *p)
       else if( !strncmp(token->v, GAL_ARITHMETIC_SET_PREFIX,
                         GAL_ARITHMETIC_SET_PREFIX_LENGTH) )
         gal_arithmetic_set_name(&p->setprm, token->v);
+      else if( (col=gal_arithmetic_load_col(token->v, cp->searchin,
+                                            cp->ignorecase,
+                                            cp->minmapsize,
+                                            cp->quietmmap))!=NULL )
+        operands_add(p, NULL, col);
       else if(    gal_array_file_recognized(token->v)
                || gal_arithmetic_set_is_name(p->setprm.named, token->v) )
         operands_add(p, token->v, NULL);
@@ -1517,12 +1752,13 @@ reversepolish(struct arithmeticparams *p)
           data->minmapsize=p->cp.minmapsize;
           operands_add(p, NULL, data);
         }
+
       /* Last option is an operator: the program will abort if the token
          isn't an operator. */
       else
         {
-          operator=arithmetic_set_operator(token->v, &num_operands);
-          arithmetic_operator_run(p, operator, token->v, num_operands);
+          operator=arithmetic_set_operator(token->v, &num_operands, &inlib);
+          arithmetic_operator_run(p, operator, token->v, num_operands, inlib);
         }
 
       /* Increment the token counter. */
@@ -1538,7 +1774,7 @@ reversepolish(struct arithmeticparams *p)
 
   /* If there is more than one node in the operands stack then the user has
      given too many operands which is an error. */
-  if(p->operands->next!=NULL)
+  if(p->writeall==0 && p->operands->next!=NULL)
     error(EXIT_FAILURE, 0, "too many operands");
 
 
@@ -1547,34 +1783,18 @@ reversepolish(struct arithmeticparams *p)
      read the contents of the file and put the resulting dataset into the
      operands 'data' element. This can happen for example if no operators
      are called and there is only one filename as an argument (which can
-     happen in scripts). */
-  if(p->operands->data==NULL && p->operands->filename)
-    {
-      /* Read the desired image and report it if necessary. */
-      hdu=p->operands->hdu;
-      filename=p->operands->filename;
-      if( gal_fits_file_recognized(filename) )
-        {
-          /* Read the data, note that the WCS has already been set. */
-          p->operands->data=gal_array_read_one_ch(filename, hdu, NULL,
-                                                  p->cp.minmapsize,
-                                                  p->cp.quietmmap);
-          data=p->operands->data;
-          data->ndim=gal_dimension_remove_extra(data->ndim, data->dsize, NULL);
-          if(!p->cp.quiet) printf(" - %s (hdu %s) is read.\n", filename, hdu);
-        }
-      else
-        error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s to fix "
-              "the problem. While 'operands->data' is NULL, the filename "
-              "('%s') is not recognized as a FITS file", __func__,
-              PACKAGE_BUGREPORT, filename);
-    }
+     happen in scripts).*/
+  for(otmp=p->operands; otmp!=NULL; otmp=otmp->next)
+    if(otmp->data==NULL && otmp->filename)
+      arithmetic_final_read_file(p, otmp);
 
 
   /* If the final data structure has more than one element, write it as a
-     FITS file. Otherwise, print it in the standard output. */
-  data=p->operands->data;
-  if(data->size==1 && data->ndim==1 && p->onedasimage==0)
+     FITS file. Otherwise, if the user didn't call '--output', print it in
+     the standard output. */
+  data=arithmetic_final_data(p);
+  if(data->next==NULL && data->size==1 && data->ndim==1
+     && p->outnamerequested==0)
     {
       /* Make the string to print the number. */
       printnum=gal_type_to_string(data->array, data->type, 0);
@@ -1585,15 +1805,29 @@ reversepolish(struct arithmeticparams *p)
     }
   else
     {
+      /* If the user has requested a name, units or comments for the FITS
+         file, insert them into the dataset here. */
+      if(p->metaname)
+        { if(data->name) free(data->name);
+          gal_checkset_allocate_copy(p->metaname, &data->name); }
+      if(p->metaunit)
+        { if(data->unit) free(data->unit);
+          gal_checkset_allocate_copy(p->metaunit, &data->unit); }
+      if(p->metacomment)
+        { if(data->comment) free(data->comment);
+          gal_checkset_allocate_copy(p->metacomment, &data->comment); }
+
       /* Put a copy of the WCS structure from the reference image, it
          will be freed while freeing 'data'. */
-      data->wcs=p->refdata.wcs;
       if(data->ndim==1 && p->onedasimage==0)
         gal_table_write(data, NULL, NULL, p->cp.tableformat,
                         p->onedonstdout ? NULL : p->cp.output,
                         "ARITHMETIC", 0);
       else
-        gal_fits_img_write(data, p->cp.output, NULL, PROGRAM_NAME);
+        for(tmp=data; tmp!=NULL; tmp=tmp->next)
+          gal_fits_img_write(tmp, p->cp.output, NULL, PROGRAM_NAME);
+
+      /* Let the user know that the job is done. */
       if(!p->cp.quiet)
         printf(" - Write (final): %s\n", p->cp.output);
     }
