@@ -5,7 +5,7 @@ ConvertType is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -336,8 +336,9 @@ convertt(struct converttparams *p)
 
     /* Plain text: only one channel is acceptable. */
     case OUT_FORMAT_TXT:
-      gal_checkset_writable_remove(p->cp.output, 0, p->cp.dontdelete);
-      gal_txt_write(p->chll, NULL, NULL, p->cp.output, 0);
+      gal_checkset_writable_remove(p->cp.output, p->inputnames->v, 0,
+                                   p->cp.dontdelete);
+      gal_txt_write(p->chll, NULL, NULL, p->cp.output, 0, 1);
       break;
 
     /* JPEG: */
@@ -350,14 +351,15 @@ convertt(struct converttparams *p)
     case OUT_FORMAT_EPS:
       if(p->colormap) color_map_prepare(p); else convertt_scale_to_uchar(p);
       gal_eps_write(p->chll, p->cp.output, p->widthincm, p->borderwidth,
-                    p->hex, p->forcemin || p->forcemax, 0);
+                    p->bordercolor, p->hex, p->forcemin || p->forcemax,
+                    0, p->marks);
       break;
 
     /* PDF */
     case OUT_FORMAT_PDF:
       if(p->colormap) color_map_prepare(p); else convertt_scale_to_uchar(p);
       gal_pdf_write(p->chll, p->cp.output, p->widthincm, p->borderwidth,
-                    p->forcemin || p->forcemax);
+                    p->bordercolor, p->forcemin || p->forcemax, p->marks);
       break;
 
     /* Not recognized. */
