@@ -5,7 +5,7 @@ ConvertType is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2022 Free Software Foundation, Inc.
+Copyright (C) 2016-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -418,7 +418,7 @@ ui_show_fonts(struct converttparams *p)
 
   /* Set the PDF and PS file names. */
   outpdf = gal_checkset_automatic_output(&p->cp, p->cp.output, "-fonts.pdf");
-  gal_checkset_writable_remove(outpdf, 0, p->cp.dontdelete);
+  gal_checkset_writable_remove(outpdf, p->cp.output, 0, p->cp.dontdelete);
   outps=gal_checkset_automatic_output(&p->cp, outpdf, ".ps");
 
   /* This command was taken from the 'ps2pdfwr' installed script that comes
@@ -1231,7 +1231,9 @@ ui_set_output(struct converttparams *p)
     }
 
   /* Check if the output already exists and remove it if allowed. */
-  gal_checkset_writable_remove(cp->output, 0, cp->dontdelete);
+  gal_checkset_writable_remove(cp->output,
+                               p->inputnames ? p->inputnames->v : NULL, 0,
+                               cp->dontdelete);
 }
 
 
