@@ -5,7 +5,7 @@ Convolve is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -561,15 +561,15 @@ ui_preparations(struct convolveparams *p)
      argument) is the blurry image. */
   if(p->makekernel)
     {
+      /* Read the kernel. */
+      ui_read_kernel(p);
+
       /* Currently this is not implemented in 1D. */
       if(p->kernel->ndim==1)
         error(EXIT_FAILURE, 0, "'--makekernel' is currently not available "
               "on 1D datasets");
       else
         {
-          /* Read in the kernel array. */
-          ui_read_kernel(p);
-
           /* Make sure the size of the kernel is the same as the input */
           if( p->input->dsize[0]!=p->kernel->dsize[0]
               || p->input->dsize[1]!=p->kernel->dsize[1] )
@@ -657,18 +657,19 @@ ui_preparations(struct convolveparams *p)
   /* Set the output name if the user hasn't set it. */
   if(cp->output==NULL)
     cp->output=gal_checkset_automatic_output(cp, p->filename, outsuffix);
-  gal_checkset_writable_remove(cp->output, 0, cp->dontdelete);
+  gal_checkset_writable_remove(cp->output, p->filename, 0, cp->dontdelete);
   if(p->checkfreqsteps)
     {
       p->freqstepsname=gal_checkset_automatic_output(cp, p->filename,
                                                      "_freqsteps.fits");
-      gal_checkset_writable_remove(p->freqstepsname, 0, cp->dontdelete);
+      gal_checkset_writable_remove(p->freqstepsname, p->filename, 0,
+                                   cp->dontdelete);
     }
   if(cp->tl.checktiles)
     {
       cp->tl.tilecheckname=gal_checkset_automatic_output(cp, p->filename,
                                                          "_tiled.fits");
-      gal_checkset_writable_remove(cp->tl.tilecheckname, 0,
+      gal_checkset_writable_remove(cp->tl.tilecheckname, p->filename, 0,
                                    cp->dontdelete);
     }
 }
