@@ -5,7 +5,7 @@ CosmicCalculator is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2019, Free Software Foundation, Inc.
+Copyright (C) 2016-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -44,6 +44,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "velocity",
+      UI_KEY_VELOCITY,
+      "FLT",
+      0,
+      "Velocity of interest in km/s.",
+      GAL_OPTIONS_GROUP_INPUT,
+      &p->velocity,
+      GAL_TYPE_FLOAT64,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "obsline",
       UI_KEY_OBSLINE,
@@ -114,10 +127,12 @@ struct argp_option program_options[] =
 
 
 
+
+    /* Basic cosmology */
     {
       0, 0, 0, 0,
-      "Specific calculations",
-      UI_GROUP_SPECIFIC
+      "Basic cosmology calculations",
+      UI_GROUP_BASIC
     },
     {
       "usedredshift",
@@ -125,7 +140,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Used redshift in this run.",
-      UI_GROUP_SPECIFIC,
+      UI_GROUP_BASIC,
       &p->specific,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -139,7 +154,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Age of universe now (Ga: Giga Annum).",
-      UI_GROUP_SPECIFIC,
+      UI_GROUP_BASIC,
       &p->specific,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -153,7 +168,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Critical density now (g/cm^3).",
-      UI_GROUP_SPECIFIC,
+      UI_GROUP_BASIC,
       &p->specific,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -167,7 +182,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Proper distance to z (Mpc).",
-      UI_GROUP_SPECIFIC,
+      UI_GROUP_BASIC,
       &p->specific,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -181,7 +196,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Angular diameter distance (Mpc).",
-      UI_GROUP_SPECIFIC,
+      UI_GROUP_BASIC,
       &p->specific,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -195,7 +210,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Tangential dist. covered by 1arcsec at z (kpc).",
-      UI_GROUP_SPECIFIC,
+      UI_GROUP_BASIC,
       &p->specific,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -209,7 +224,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Luminosity distance to z (Mpc).",
-      UI_GROUP_SPECIFIC,
+      UI_GROUP_BASIC,
       &p->specific,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -223,7 +238,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Distance modulus at z (no units).",
-      UI_GROUP_SPECIFIC,
+      UI_GROUP_BASIC,
       &p->specific,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -237,7 +252,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Conversion to absolute magnitude (no unit).",
-      UI_GROUP_SPECIFIC,
+      UI_GROUP_BASIC,
       &p->specific,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -251,7 +266,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Age of universe at z (Ga: Giga Annum).",
-      UI_GROUP_SPECIFIC,
+      UI_GROUP_BASIC,
       &p->specific,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -265,7 +280,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Look back time to z (Ga: Giga Annum).",
-      UI_GROUP_SPECIFIC,
+      UI_GROUP_BASIC,
       &p->specific,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -279,7 +294,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Critical density at z (g/cm^3).",
-      UI_GROUP_SPECIFIC,
+      UI_GROUP_BASIC,
       &p->specific,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -293,7 +308,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Comoving volume (4pi str) to z (Mpc^3).",
-      UI_GROUP_SPECIFIC,
+      UI_GROUP_BASIC,
       &p->specific,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -301,13 +316,77 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       ui_add_to_single_value,
     },
+    {
+      "usedvelocity",
+      UI_KEY_USEDVELOCITY,
+      0,
+      0,
+      "Used velocity (in km/s) for this run.",
+      UI_GROUP_BASIC,
+      &p->specific,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_add_to_single_value,
+    },
+
+
+
+
+
+    /* Spectral line options. */
+    {
+      0, 0, 0, 0,
+      "Spectral lines",
+      UI_GROUP_SPECTRAL_LINES
+    },
+    {
+      "listlines",
+      UI_KEY_LISTLINES,
+      0,
+      0,
+      "List pre-defined lines at rest frame.",
+      UI_GROUP_SPECTRAL_LINES,
+      &p->listlines,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "listlinesatz",
+      UI_KEY_LISTLINESATZ,
+      0,
+      0,
+      "List pre-defined lines at the given redshift.",
+      UI_GROUP_SPECTRAL_LINES,
+      &p->listlinesatz,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "lineunit",
+      UI_KEY_LINEUNIT,
+      "STR",
+      0,
+      "Unit ('angstrom', 'nm', 'microm' or 'm').",
+      UI_GROUP_SPECTRAL_LINES,
+      &p->lineunit,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+    },
     {
       "lineatz",
       UI_KEY_LINEATZ,
       "STR/FLT",
       0,
-      "Wavelength of given line at chosen redshift",
-      UI_GROUP_SPECIFIC,
+      "Wavelength of line (name or wavelength) at z.",
+      UI_GROUP_SPECTRAL_LINES,
       &p->specific,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -316,8 +395,6 @@ struct argp_option program_options[] =
       ui_add_to_single_value,
     },
 
-
-
     {0}
   };
 
