@@ -5,7 +5,8 @@ Fits is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2021, Free Software Foundation, Inc.
+     Pedram Ashofteh-Ardakani <pedramardakani@pm.me>
+Copyright (C) 2017-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -40,6 +41,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include "main.h"
 
 #include "fits.h"
+#include "meta.h"
 #include "keywords.h"
 
 
@@ -88,10 +90,12 @@ fits_print_extension_info(struct fitsparams *p)
 {
   uint16_t *ui16;
   fitsfile *fptr;
+  gal_data_t *unitscol;
+  char bunit[FLEN_VALUE];
   size_t i, numext, *dsize, ndim;
-  int hascomments=0, hasblankname=0;
-  char **tstra, **estra, **sstra, **cmstra;
   int j, nc, numhdu, hdutype, status=0, type;
+  char **tstra, **estra, **sstra, **cmstra, **ustra;
+  int hascomments=0, hasblankname=0, hasblankunits=0;
   gal_data_t *tmp, *cols=NULL, *sizecol, *commentscol;
   char *msg, *tstr=NULL, sstr[1000], extname[FLEN_VALUE];
 
@@ -99,7 +103,7 @@ fits_print_extension_info(struct fitsparams *p)
   /* Open the FITS file and read the first extension type, upon moving to
      the next extension, we will read its type, so for the first we will
      need to do it explicitly. */
-  fptr=gal_fits_hdu_open(p->input->v, "0", READONLY);
+  fptr=gal_fits_hdu_open(p->input->v, "0", READONLY, 1);
   if (fits_get_hdu_type(fptr, &hdutype, &status) )
     gal_fits_io_error(status, "reading first extension");
 
@@ -114,6 +118,8 @@ fits_print_extension_info(struct fitsparams *p)
      linked list). */
   gal_list_data_add_alloc(&cols, NULL, GAL_TYPE_STRING, 1, &numext, NULL, 1,
                           -1, 1, "HDU_COMMENT", "note", "Possible comment");
+  gal_list_data_add_alloc(&cols, NULL, GAL_TYPE_STRING, 1, &numext, NULL, 1,
+                          -1, 1, "HDU_UNITS", "units", "Units of the pixels");
   gal_list_data_add_alloc(&cols, NULL, GAL_TYPE_STRING, 1, &numext, NULL, 1,
                           -1, 1, "HDU_SIZE", "name", "Size of image or table "
                           "number of rows and columns.");
@@ -136,7 +142,10 @@ fits_print_extension_info(struct fitsparams *p)
   sizecol = cols->next->next->next;
   sstra = sizecol->array;
 
-  commentscol = cols->next->next->next->next;
+  unitscol = cols->next->next->next->next;
+  ustra = unitscol->array;
+
+  commentscol = cols->next->next->next->next->next;
   cmstra = commentscol->array;
 
   cols->next->disp_width=15;
@@ -189,6 +198,31 @@ fits_print_extension_info(struct fitsparams *p)
       status=0;
 
 
+      /* Read the extension units (only for images). */
+      if(hdutype==IMAGE_HDU)
+        fits_read_keyword(fptr, "BUNIT", bunit, NULL, &status);
+      switch(status)
+        {
+        case 0:
+          /* For tables, we don't bother reading 'BUNIT', because it is
+             meaningless even if it exists (each colum has its separate
+             units). So 'status' will be zero in this case.*/
+          if(hdutype==IMAGE_HDU)
+            gal_fits_key_clean_str_value(bunit);
+          else /* Its a table HDU. */
+            { sprintf(bunit, "%s", GAL_BLANK_STRING); hasblankunits=1; }
+          break;
+
+        case KEY_NO_EXIST:
+          sprintf(bunit, "%s", GAL_BLANK_STRING); hasblankunits=1;
+          break;
+
+        default:
+          gal_fits_io_error(status, "reading BUNIT keyword");
+        }
+      status=0;
+
+
       /* Check if its a healpix grid. */
       if( gal_fits_hdu_is_healpix(fptr) )
         {
@@ -229,6 +263,7 @@ fits_print_extension_info(struct fitsparams *p)
             case 1: gal_checkset_allocate_copy(extname, estra+i); break;
             case 2: gal_checkset_allocate_copy(tstr, tstra+i);    break;
             case 3: gal_checkset_allocate_copy(sstr, sstra+i);    break;
+            case 4: gal_checkset_allocate_copy(bunit, ustra+i);   break;
             }
           ++j;
         }
@@ -249,7 +284,7 @@ fits_print_extension_info(struct fitsparams *p)
   /* If there weren't any comments, don't print the comment column. */
   if(hascomments==0)
     {
-      sizecol->next=NULL;
+      unitscol->next=NULL;
       gal_data_free(commentscol);
     }
 
@@ -262,13 +297,18 @@ fits_print_extension_info(struct fitsparams *p)
       printf(" Column 2: Name ('EXTNAME' in FITS standard, usable with "
              "'--hdu').\n");
       if(hasblankname)
-        printf("           ('%s' means that no name is specified for this "
-               "HDU)\n", GAL_BLANK_STRING);
+        printf("           ('%s': no name in HDU metadata)\n",
+               GAL_BLANK_STRING);
       printf(" Column 3: Image data type or 'table' format (ASCII or "
              "binary).\n");
       printf(" Column 4: Size of data in HDU.\n");
+      printf(" Column 5: Units of data in HDU (only images, for tables "
+             "use 'asttable -i').\n");
+      if(hasblankunits)
+        printf("           ('%s': no unit in HDU metadata, or "
+               "HDU is a table)\n", GAL_BLANK_STRING);
       if(hascomments)
-        printf(" Column 5: Comments about the HDU (e.g., if its HEALpix, or "
+        printf(" Column 6: Comments about the HDU (e.g., if its HEALpix, or "
                "etc).\n");
       printf("-----\n");
     }
@@ -287,7 +327,7 @@ fits_hdu_number(struct fitsparams *p)
   int numhdu, status=0;
 
   /* Read the first extension (necessary for reading the rest). */
-  fptr=gal_fits_hdu_open(p->input->v, "0", READONLY);
+  fptr=gal_fits_hdu_open(p->input->v, "0", READONLY, 1);
 
   /* Get the number of HDUs. */
   if( fits_get_num_hdus(fptr, &numhdu, &status) )
@@ -314,13 +354,12 @@ fits_datasum(struct fitsparams *p)
 
 
 
-static void
-fits_pixelscale(struct fitsparams *p)
+struct wcsprm *
+fits_read_check_wcs(struct fitsparams *p, size_t *ndim,
+                    char *operation)
 {
   int nwcs=0;
-  size_t i, ndim=0;
   struct wcsprm *wcs;
-  double multip, *pixelscale;
 
   /* Read the desired WCS. */
   wcs=gal_wcs_read(p->input->v, p->cp.hdu, p->cp.wcslinearmatrix,
@@ -328,11 +367,26 @@ fits_pixelscale(struct fitsparams *p)
 
   /* If a WCS doesn't exist, let the user know and return. */
   if(wcs)
-    ndim=wcs->naxis;
+    *ndim=wcs->naxis;
   else
     error(EXIT_FAILURE, 0, "%s (hdu %s): no WCS could be read by WCSLIB, "
-          "hence the pixel-scale cannot be determined", p->input->v,
-          p->cp.hdu);
+          "hence the %s cannot be determined", p->input->v,
+          p->cp.hdu, operation);
+
+  /* Return the WCS. */
+  return wcs;
+}
+
+
+
+
+
+static void
+fits_pixelscale(struct fitsparams *p)
+{
+  size_t i, ndim=0;
+  double multip, *pixelscale;
+  struct wcsprm *wcs=fits_read_check_wcs(p, &ndim, "pixel scale");
 
   /* Calculate the pixel-scale in each dimension. */
   pixelscale=gal_wcs_pixel_scale(wcs);
@@ -435,6 +489,42 @@ fits_pixelscale(struct fitsparams *p)
 
 
 
+static void
+fits_pixelarea(struct fitsparams *p)
+{
+  double area;
+  size_t i, ndim=0;
+  double *pixelscale;
+  struct wcsprm *wcs=fits_read_check_wcs(p, &ndim, "pixel area");
+
+  /* Basic sanity checks. */
+  if(ndim!=2)
+    error(EXIT_FAILURE, 0, "the pixel area can only be calculated "
+          "for two dimensional data (images). The given input has "
+          "%zu dimensions", ndim);
+  for(i=0;i<ndim;++i)
+    if( strcmp(wcs->cunit[i], "deg") )
+      error(EXIT_FAILURE, 0, "%s (hdu: %s): dimension %zu doesn't "
+            "have a unit of degrees (value to 'CUNIT%zu'), but "
+            "'%s'! In order to use '--pixelareaarcsec2', the units "
+            "of the WCS have to be in degrees", p->input->v,
+            p->cp.hdu, i+1, i+1, wcs->cunit[i]);
+
+  /* Calculate the pixel-scale in each dimension. */
+  pixelscale=gal_wcs_pixel_scale(wcs);
+
+  /* Calcluate the pixel area (currently only in arcsec^N) */
+  area=1;
+  for(i=0;i<ndim;++i) area *= pixelscale[i]*3600;
+  printf("%g\n", area);
+
+  /* Clean up. */
+  wcsfree(wcs);
+  free(pixelscale);
+}
+
+
+
 static void
 fits_skycoverage(struct fitsparams *p)
 {
@@ -608,6 +698,9 @@ fits_certain_hdu(struct fitsparams *p, int list1has0,
   status=0;
   fits_close_file(fptr, &status);
 
+  /* Reverse the list so they are in the same order as they were found. */
+  gal_list_str_reverse(&names);
+
   /* Print the result. */
   if( list1has0 )
     {
@@ -670,7 +763,7 @@ fits_hdu_remove(struct fitsparams *p, int *r)
       hdu=gal_list_str_pop(&p->remove);
 
       /* Open the FITS file at the specified HDU. */
-      fptr=gal_fits_hdu_open(p->input->v, hdu, READWRITE);
+      fptr=gal_fits_hdu_open(p->input->v, hdu, READWRITE, 1);
 
       /* Delete the extension. */
       if( fits_delete_hdu(fptr, &hdutype, &status) )
@@ -733,7 +826,7 @@ fits_hdu_copy(struct fitsparams *p, int cut1_copy0, int *r)
 
       /* Open the FITS file at the specified HDU. */
       in=gal_fits_hdu_open(p->input->v, hdu,
-                           cut1_copy0 ? READWRITE : READONLY);
+                           cut1_copy0 ? READWRITE : READONLY, 1);
 
       /* If the output isn't opened yet, open it.  */
       if(out==NULL)
@@ -775,24 +868,27 @@ fits(struct fitsparams *p)
 
   switch(p->mode)
     {
+    /* Functions creating meta-images, in other words, images conveying
+       information about the given image. */
+    case FITS_MODE_META: meta_pixelareaonwcs(p); break;
+
     /* Keywords, we have a separate set of functions in 'keywords.c'. */
-    case FITS_MODE_KEY:
-      r=keywords(p);
-      break;
+    case FITS_MODE_KEY: r=keywords(p); break;
 
     /* HDU, functions defined here. */
     case FITS_MODE_HDU:
 
       /* Options that must be called alone. */
-      if(p->numhdus) fits_hdu_number(p);
-      else if(p->datasum)       fits_datasum(p);
-      else if(p->pixelscale)    fits_pixelscale(p);
-      else if(p->skycoverage)   fits_skycoverage(p);
-      else if(p->hasimagehdu)   fits_certain_hdu(p, 0, 0);
-      else if(p->hastablehdu)   fits_certain_hdu(p, 0, 1);
-      else if(p->listimagehdus) fits_certain_hdu(p, 1, 0);
-      else if(p->listtablehdus) fits_certain_hdu(p, 1, 1);
-      else if(p->listallhdus)   fits_list_all_hdus(p);
+      if(p->numhdus)               fits_hdu_number(p);
+      else if(p->datasum)          fits_datasum(p);
+      else if(p->pixelscale)       fits_pixelscale(p);
+      else if(p->pixelareaarcsec2) fits_pixelarea(p);
+      else if(p->skycoverage)      fits_skycoverage(p);
+      else if(p->hasimagehdu)      fits_certain_hdu(p, 0, 0);
+      else if(p->hastablehdu)      fits_certain_hdu(p, 0, 1);
+      else if(p->listimagehdus)    fits_certain_hdu(p, 1, 0);
+      else if(p->listtablehdus)    fits_certain_hdu(p, 1, 1);
+      else if(p->listallhdus)      fits_list_all_hdus(p);
 
       /* Options that can be called together. */
       else
