@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2022 Free Software Foundation, Inc.
+Copyright (C) 2016-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -258,19 +258,6 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
-    {
-      "spectrum",
-      UI_KEY_SPECTRUM,
-      0,
-      0,
-      "Object spectrum for cube (3D) datasets.",
-      GAL_OPTIONS_GROUP_OUTPUT,
-      &p->spectrum,
-      GAL_OPTIONS_NO_ARG_TYPE,
-      GAL_OPTIONS_RANGE_0_OR_1,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
     {
       "inbetweenints",
       UI_KEY_INBETWEENINTS,
@@ -414,11 +401,11 @@ struct argp_option program_options[] =
       UI_GROUP_OTHERSETTINGS
     },
     {
-      "fracmax",
+      "frac-max",
       UI_KEY_FRACMAX,
       "FLT[,FLT]",
       0,
-      "Fraction(s) in --fracmaxarea1 or --fracmaxarea2.",
+      "Fraction(s) in --frac-max* options.",
       UI_GROUP_OTHERSETTINGS,
       &p->fracmax,
       GAL_TYPE_STRING,
@@ -432,7 +419,7 @@ struct argp_option program_options[] =
       UI_KEY_SPATIALRESOLUTION,
       "FLT",
       0,
-      "Spatial resolution (for surface brightness error).",
+      "Spatial resolution (for surf. brightness err).",
       UI_GROUP_OTHERSETTINGS,
       &p->spatialresolution,
       GAL_TYPE_FLOAT32,
@@ -467,7 +454,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "objid",
+      "obj-id",
       UI_KEY_OBJID,
       0,
       0,
@@ -481,7 +468,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "hostobjid",
+      "host-obj-id",
       UI_KEY_HOSTOBJID,
       0,
       0,
@@ -495,7 +482,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "idinhostobj",
+      "id-in-host-obj",
       UI_KEY_IDINHOSTOBJ,
       0,
       0,
@@ -516,7 +503,7 @@ struct argp_option program_options[] =
     /* Position related columns (pixel). */
     {
       0, 0, 0, 0,
-      "Positional columns (pixel)",
+      "Positional (pixel/image) measurements",
       UI_GROUP_COLUMNS_POSITION_PIXEL
     },
     {
@@ -562,7 +549,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "geox",
+      "geo-x",
       UI_KEY_GEOX,
       0,
       0,
@@ -576,7 +563,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "geoy",
+      "geo-y",
       UI_KEY_GEOY,
       0,
       0,
@@ -590,7 +577,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "geoz",
+      "geo-z",
       UI_KEY_GEOZ,
       0,
       0,
@@ -604,11 +591,11 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "minvx",
-      UI_KEY_MINVX,
+      "min-val-x",
+      UI_KEY_MINVALX,
       0,
       0,
-      "Minimum value's X axis position",
+      "Minimum value's X axis position.",
       UI_GROUP_COLUMNS_POSITION_PIXEL,
       0,
       GAL_TYPE_INVALID,
@@ -618,11 +605,11 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "maxvx",
-      UI_KEY_MAXVX,
+      "max-val-x",
+      UI_KEY_MAXVALX,
       0,
       0,
-      "Maximum value's X axis position",
+      "Maximum value's X axis position.",
       UI_GROUP_COLUMNS_POSITION_PIXEL,
       0,
       GAL_TYPE_INVALID,
@@ -632,11 +619,11 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "minvy",
-      UI_KEY_MINVY,
+      "min-val-y",
+      UI_KEY_MINVALY,
       0,
       0,
-      "Minimum value's Y axis position",
+      "Minimum value's Y axis position.",
       UI_GROUP_COLUMNS_POSITION_PIXEL,
       0,
       GAL_TYPE_INVALID,
@@ -646,11 +633,11 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "maxvy",
-      UI_KEY_MAXVY,
+      "max-val-y",
+      UI_KEY_MAXVALY,
       0,
       0,
-      "Maximum value's Y axis position",
+      "Maximum value's Y axis position.",
       UI_GROUP_COLUMNS_POSITION_PIXEL,
       0,
       GAL_TYPE_INVALID,
@@ -660,11 +647,11 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "minvz",
-      UI_KEY_MINVZ,
+      "min-val-z",
+      UI_KEY_MINVALZ,
       0,
       0,
-      "Minimum value's Z axis position",
+      "Minimum value's Z axis position.",
       UI_GROUP_COLUMNS_POSITION_PIXEL,
       0,
       GAL_TYPE_INVALID,
@@ -674,8 +661,8 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "maxvz",
-      UI_KEY_MAXVZ,
+      "max-val-z",
+      UI_KEY_MAXVALZ,
       0,
       0,
       "Maximum value's Z axis position",
@@ -688,7 +675,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "minx",
+      "min-x",
       UI_KEY_MINX,
       0,
       0,
@@ -702,7 +689,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "maxx",
+      "max-x",
       UI_KEY_MAXX,
       0,
       0,
@@ -716,7 +703,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "miny",
+      "min-y",
       UI_KEY_MINY,
       0,
       0,
@@ -730,7 +717,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "maxy",
+      "max-y",
       UI_KEY_MAXY,
       0,
       0,
@@ -744,7 +731,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "minz",
+      "min-z",
       UI_KEY_MINZ,
       0,
       0,
@@ -758,7 +745,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "maxz",
+      "max-z",
       UI_KEY_MAXZ,
       0,
       0,
@@ -772,7 +759,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsx",
+      "clumps-x",
       UI_KEY_CLUMPSX,
       0,
       0,
@@ -786,7 +773,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsy",
+      "clumps-y",
       UI_KEY_CLUMPSY,
       0,
       0,
@@ -800,7 +787,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsz",
+      "clumps-z",
       UI_KEY_CLUMPSZ,
       0,
       0,
@@ -814,7 +801,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsgeox",
+      "clumps-geo-x",
       UI_KEY_CLUMPSGEOX,
       0,
       0,
@@ -828,7 +815,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsgeoy",
+      "clumps-geo-y",
       UI_KEY_CLUMPSGEOY,
       0,
       0,
@@ -842,7 +829,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsgeoz",
+      "clumps-geo-z",
       UI_KEY_CLUMPSGEOZ,
       0,
       0,
@@ -863,7 +850,7 @@ struct argp_option program_options[] =
     /* Position related columns (WCS). */
     {
       0, 0, 0, 0,
-      "Positional columns (WCS)",
+      "Positional (WCS) measurements",
       UI_GROUP_COLUMNS_POSITION_WCS
     },
     {
@@ -937,7 +924,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "geow1",
+      "geo-w1",
       UI_KEY_GEOW1,
       0,
       0,
@@ -951,7 +938,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "geow2",
+      "geo-w2",
       UI_KEY_GEOW2,
       0,
       0,
@@ -965,7 +952,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "geow3",
+      "geo-w3",
       UI_KEY_GEOW2,
       0,
       0,
@@ -979,7 +966,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsw1",
+      "clumps-w1",
       UI_KEY_CLUMPSW1,
       0,
       0,
@@ -993,7 +980,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsw2",
+      "clumps-w2",
       UI_KEY_CLUMPSW2,
       0,
       0,
@@ -1007,7 +994,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsw3",
+      "clumps-w3",
       UI_KEY_CLUMPSW3,
       0,
       0,
@@ -1021,7 +1008,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsgeow1",
+      "clumps-geo-w1",
       UI_KEY_CLUMPSGEOW1,
       0,
       0,
@@ -1035,7 +1022,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsgeow2",
+      "clumps-geo-w2",
       UI_KEY_CLUMPSGEOW2,
       0,
       0,
@@ -1049,7 +1036,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsgeow3",
+      "clumps-geo-w3",
       UI_KEY_CLUMPSGEOW3,
       0,
       0,
@@ -1070,15 +1057,15 @@ struct argp_option program_options[] =
     /* Brightness/pixel-value related columns. */
     {
       0, 0, 0, 0,
-      "Brightness/magnitude related columns",
+      "Brightness/magnitude (only using pixel value/error) measurements",
       UI_GROUP_COLUMNS_BRIGHTNESS
     },
     {
-      "brightness",
-      UI_KEY_BRIGHTNESS,
+      "sum",
+      UI_KEY_SUM,
       0,
       0,
-      "Brightness (sum of pixel values).",
+      "Sum of pixel values in each label.",
       UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
@@ -1088,11 +1075,11 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "brightnesserr",
-      UI_KEY_BRIGHTNESSERR,
+      "sum-error",
+      UI_KEY_SUMERROR,
       0,
       0,
-      "Error (1-sigma) in measuring brightness.",
+      "Error (1-sigma) in measuring sum.",
       UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
@@ -1102,8 +1089,8 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpbrightness",
-      UI_KEY_CLUMPSBRIGHTNESS,
+      "clumps-sum",
+      UI_KEY_CLUMPSSUM,
       0,
       0,
       "Brightness of clumps in an object.",
@@ -1116,11 +1103,11 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "brightnessnoriver",
-      UI_KEY_BRIGHTNESSNORIVER,
+      "sum-no-river",
+      UI_KEY_SUMNORIVER,
       0,
       0,
-      "Sky (not river) subtracted clump brightness.",
+      "Sky (not river) subtracted clump sum.",
       UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
@@ -1200,8 +1187,8 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "magnitudeerr",
-      UI_KEY_MAGNITUDEERR,
+      "magnitude-error",
+      UI_KEY_MAGNITUDEERROR,
       0,
       0,
       "Magnitude error of objects or clumps.",
@@ -1214,7 +1201,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsmagnitude",
+      "clumps-magnitude",
       UI_KEY_CLUMPSMAGNITUDE,
       0,
       0,
@@ -1242,7 +1229,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "upperlimitmag",
+      "upperlimit-mag",
       UI_KEY_UPPERLIMITMAG,
       0,
       0,
@@ -1256,21 +1243,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "upperlimitsb",
-      UI_KEY_UPPERLIMITSB,
-      0,
-      0,
-      "Upper-limit surface brightness (mag/arcsec^2).",
-      UI_GROUP_COLUMNS_BRIGHTNESS,
-      0,
-      GAL_TYPE_INVALID,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      ui_column_codes_ll
-    },
-    {
-      "upperlimitonesigma",
+      "upperlimit-onesigma",
       UI_KEY_UPPERLIMITONESIGMA,
       0,
       0,
@@ -1284,7 +1257,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "upperlimitsigma",
+      "upperlimit-sigma",
       UI_KEY_UPPERLIMITSIGMA,
       0,
       0,
@@ -1298,7 +1271,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "upperlimitquantile",
+      "upperlimit-quantile",
       UI_KEY_UPPERLIMITQUANTILE,
       0,
       0,
@@ -1312,7 +1285,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "upperlimitskew",
+      "upperlimit-skew",
       UI_KEY_UPPERLIMITSKEW,
       0,
       0,
@@ -1326,11 +1299,11 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "riverave",
-      UI_KEY_RIVERAVE,
+      "river-mean",
+      UI_KEY_RIVERMEAN,
       0,
       0,
-      "Average river value surrounding a clump.",
+      "Mean river value surrounding a clump.",
       UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
@@ -1340,7 +1313,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "rivernum",
+      "river-num",
       UI_KEY_RIVERNUM,
       0,
       0,
@@ -1382,7 +1355,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "skystd",
+      "sky-std",
       UI_KEY_SKYSTD,
       0,
       0,
@@ -1455,14 +1428,138 @@ struct argp_option program_options[] =
 
 
 
+
+    /* Brightness/pixel-value related columns. */
+    {
+      0, 0, 0, 0,
+      "Surface brightness measurements (all: mag/arcsec^2)",
+      UI_GROUP_COLUMNS_SURFACEBRIGHTNESS
+    },
+    {
+      "sb",
+      UI_KEY_SB,
+      0,
+      0,
+      "Surface brightness.",
+      UI_GROUP_COLUMNS_SURFACEBRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "sb-error",
+      UI_KEY_SBERROR,
+      0,
+      0,
+      "Surface brightness error from STD/VAR image.",
+      UI_GROUP_COLUMNS_SURFACEBRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "upperlimit-sb",
+      UI_KEY_UPPERLIMITSB,
+      0,
+      0,
+      "Upper-limit surface brightness.",
+      UI_GROUP_COLUMNS_SURFACEBRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "sigclip-mean-sb",
+      UI_KEY_SIGCLIPMEANSB,
+      0,
+      0,
+      "Surface brightness of sigclip-mean (1 pix area).",
+      UI_GROUP_COLUMNS_SURFACEBRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "sigclip-mean-sb-delta",
+      UI_KEY_SIGCLIPMEANSBDELTA,
+      0,
+      0,
+      "sigclip-mean-sb delta from sigclip'd STD.",
+      UI_GROUP_COLUMNS_SURFACEBRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "sigclip-std-sb",
+      UI_KEY_SIGCLIPSTDSB,
+      0,
+      0,
+      "Surface brightness of sigclip-std (1 pix area).",
+      UI_GROUP_COLUMNS_SURFACEBRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "half-sum-sb",
+      UI_KEY_HALFSUMSB,
+      0,
+      0,
+      "Surface brightness within --halfsumarea.",
+      UI_GROUP_COLUMNS_SURFACEBRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "half-max-sb",
+      UI_KEY_HALFMAXSB,
+      0,
+      0,
+      "Surface brightness within half the maximum.",
+      UI_GROUP_COLUMNS_SURFACEBRIGHTNESS,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+
+
+
+
+
     /* Morphology/shape related columns. */
     {
       0, 0, 0, 0,
-      "Morphology/shape related columns",
+      "Morphology/shape (non-parametric) measurements",
       UI_GROUP_COLUMNS_MORPHOLOGY
     },
     {
-      "numclumps",
+      "num-clumps",
       UI_KEY_NUMCLUMPS,
       0,
       0,
@@ -1490,7 +1587,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "areaarcsec2",
+      "area-arcsec2",
       UI_KEY_AREAARCSEC2,
       0,
       0,
@@ -1504,11 +1601,11 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "areaminv",
-      UI_KEY_MINVNUM,
+      "area-min-val",
+      UI_KEY_MINVALNUM,
       0,
       0,
-      "Number of pixels with minimum value.",
+      "Number of pixels used in '--min-val-*'.",
       UI_GROUP_COLUMNS_POSITION_PIXEL,
       0,
       GAL_TYPE_INVALID,
@@ -1518,11 +1615,11 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "areamaxv",
-      UI_KEY_MAXVNUM,
+      "area-max-val",
+      UI_KEY_MAXVALNUM,
       0,
       0,
-      "Number of pixels with maximum value.",
+      "Number of pixels used in '--max-val-*'.",
       UI_GROUP_COLUMNS_POSITION_PIXEL,
       0,
       GAL_TYPE_INVALID,
@@ -1532,35 +1629,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "surfacebrightness",
-      UI_KEY_SURFACEBRIGHTNESS,
-      0,
-      0,
-      "Surface brightness (mag/arcsec^2).",
-      UI_GROUP_COLUMNS_MORPHOLOGY,
-      0,
-      GAL_TYPE_INVALID,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      ui_column_codes_ll
-    },
-    {
-      "sberror",
-      UI_KEY_SBERROR,
-      0,
-      0,
-      "Surface brightness error (mag/arcsec^2).",
-      UI_GROUP_COLUMNS_MORPHOLOGY,
-      0,
-      GAL_TYPE_INVALID,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      ui_column_codes_ll
-    },
-    {
-      "areaxy",
+      "area-xy",
       UI_KEY_AREAXY,
       0,
       0,
@@ -1574,7 +1643,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "clumpsarea",
+      "clumps-area",
       UI_KEY_CLUMPSAREA,
       0,
       0,
@@ -1588,7 +1657,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "weightarea",
+      "weight-area",
       UI_KEY_WEIGHTAREA,
       0,
       0,
@@ -1602,7 +1671,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "geoarea",
+      "geo-area",
       UI_KEY_GEOAREA,
       0,
       0,
@@ -1616,123 +1685,11 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "geoareaxy",
+      "geo-area-xy",
       UI_KEY_GEOAREAXY,
       0,
       0,
-      "Projected geoarea in first two dimensions.",
-      UI_GROUP_COLUMNS_MORPHOLOGY,
-      0,
-      GAL_TYPE_INVALID,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      ui_column_codes_ll
-    },
-    {
-      "semimajor",
-      UI_KEY_SEMIMAJOR,
-      0,
-      0,
-      "RMS along major axis (in pixels).",
-      UI_GROUP_COLUMNS_MORPHOLOGY,
-      0,
-      GAL_TYPE_INVALID,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      ui_column_codes_ll
-    },
-    {
-      "semiminor",
-      UI_KEY_SEMIMINOR,
-      0,
-      0,
-      "RMS along minor axis (in pixels).",
-      UI_GROUP_COLUMNS_MORPHOLOGY,
-      0,
-      GAL_TYPE_INVALID,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      ui_column_codes_ll
-    },
-    {
-      "axisratio",
-      UI_KEY_AXISRATIO,
-      0,
-      0,
-      "Flux weighted axis ratio.",
-      UI_GROUP_COLUMNS_MORPHOLOGY,
-      0,
-      GAL_TYPE_INVALID,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      ui_column_codes_ll
-    },
-    {
-      "positionangle",
-      UI_KEY_POSITIONANGLE,
-      0,
-      0,
-      "Flux weighted position angle.",
-      UI_GROUP_COLUMNS_MORPHOLOGY,
-      0,
-      GAL_TYPE_INVALID,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      ui_column_codes_ll
-    },
-    {
-      "geosemimajor",
-      UI_KEY_GEOSEMIMAJOR,
-      0,
-      0,
-      "RMS along major axis (ignoring value).",
-      UI_GROUP_COLUMNS_MORPHOLOGY,
-      0,
-      GAL_TYPE_INVALID,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      ui_column_codes_ll
-    },
-    {
-      "geosemiminor",
-      UI_KEY_GEOSEMIMINOR,
-      0,
-      0,
-      "RMS along minor axis (ignoring value).",
-      UI_GROUP_COLUMNS_MORPHOLOGY,
-      0,
-      GAL_TYPE_INVALID,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      ui_column_codes_ll
-    },
-    {
-      "geoaxisratio",
-      UI_KEY_GEOAXISRATIO,
-      0,
-      0,
-      "Geometric axis ratio.",
-      UI_GROUP_COLUMNS_MORPHOLOGY,
-      0,
-      GAL_TYPE_INVALID,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      ui_column_codes_ll
-    },
-    {
-      "geopositionangle",
-      UI_KEY_GEOPOSITIONANGLE,
-      0,
-      0,
-      "Geometric position angle.",
+      "Projected geo-area in first two dimensions.",
       UI_GROUP_COLUMNS_MORPHOLOGY,
       0,
       GAL_TYPE_INVALID,
@@ -1756,7 +1713,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "halfmaxarea",
+      "half-max-area",
       UI_KEY_HALFMAXAREA,
       0,
       0,
@@ -1770,7 +1727,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "halfmaxradius",
+      "half-max-radius",
       UI_KEY_HALFMAXRADIUS,
       0,
       0,
@@ -1784,7 +1741,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "halfmaxsum",
+      "half-max-sum",
       UI_KEY_HALFMAXSUM,
       0,
       0,
@@ -1798,21 +1755,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "halfmaxsb",
-      UI_KEY_HALFMAXSB,
-      0,
-      0,
-      "Surface brightness within half the maximum.",
-      UI_GROUP_COLUMNS_MORPHOLOGY,
-      0,
-      GAL_TYPE_INVALID,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      ui_column_codes_ll
-    },
-    {
-      "halfsumarea",
+      "half-sum-area",
       UI_KEY_HALFSUMAREA,
       0,
       0,
@@ -1826,21 +1769,7 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "halfsumsb",
-      UI_KEY_HALFSUMSB,
-      0,
-      0,
-      "Surface brightness within --halfsumarea.",
-      UI_GROUP_COLUMNS_MORPHOLOGY,
-      0,
-      GAL_TYPE_INVALID,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      ui_column_codes_ll
-    },
-    {
-      "halfsumradius",
+      "half-sum-radius",
       UI_KEY_HALFSUMRADIUS,
       0,
       0,
@@ -1854,8 +1783,8 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "fracmaxsum1",
-      UI_KEY_FRACMAXSUM1,
+      "frac-max1-sum",
+      UI_KEY_FRACMAX1SUM,
       0,
       0,
       "Sum of pixels brighter than 1st frac. of max.",
@@ -1868,8 +1797,8 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "fracmaxsum2",
-      UI_KEY_FRACMAXSUM2,
+      "frac-max2-sum",
+      UI_KEY_FRACMAX2SUM,
       0,
       0,
       "Sum of pixels brighter than 2nd frac. of max.",
@@ -1882,8 +1811,8 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "fracmaxarea1",
-      UI_KEY_FRACMAXAREA1,
+      "frac-max1-area",
+      UI_KEY_FRACMAX1AREA,
       0,
       0,
       "Area containing 1st fraction of maximum.",
@@ -1896,8 +1825,8 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "fracmaxarea2",
-      UI_KEY_FRACMAXAREA2,
+      "frac-max2-area",
+      UI_KEY_FRACMAX2AREA,
       0,
       0,
       "Area containing 2nd fraction of maximum.",
@@ -1910,8 +1839,8 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "fracmaxradius1",
-      UI_KEY_FRACMAXRADIUS1,
+      "frac-max1-radius",
+      UI_KEY_FRACMAX1RADIUS,
       0,
       0,
       "Radius calculated from --fracmaxarea1.",
@@ -1924,8 +1853,8 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "fracmaxradius2",
-      UI_KEY_FRACMAXRADIUS2,
+      "frac-max2-radius",
+      UI_KEY_FRACMAX2RADIUS,
       0,
       0,
       "Radius calculated from --fracmaxarea2.",
@@ -1942,6 +1871,265 @@ struct argp_option program_options[] =
 
 
 
+    /* Elliptical measurements columns. */
+    {
+      0, 0, 0, 0,
+      "Morphology/shape (elliptical) measurements",
+      UI_GROUP_COLUMNS_ELLIPTICAL
+    },
+    {
+      "semi-major",
+      UI_KEY_SEMIMAJOR,
+      0,
+      0,
+      "RMS along major axis (in pixels).",
+      UI_GROUP_COLUMNS_ELLIPTICAL,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "semi-minor",
+      UI_KEY_SEMIMINOR,
+      0,
+      0,
+      "RMS along minor axis (in pixels).",
+      UI_GROUP_COLUMNS_ELLIPTICAL,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "axis-ratio",
+      UI_KEY_AXISRATIO,
+      0,
+      0,
+      "Flux weighted axis ratio.",
+      UI_GROUP_COLUMNS_ELLIPTICAL,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "position-angle",
+      UI_KEY_POSITIONANGLE,
+      0,
+      0,
+      "Flux weighted position angle.",
+      UI_GROUP_COLUMNS_ELLIPTICAL,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "geo-semi-major",
+      UI_KEY_GEOSEMIMAJOR,
+      0,
+      0,
+      "Geometric RMS along major axis (ignoring value).",
+      UI_GROUP_COLUMNS_ELLIPTICAL,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "geo-semi-minor",
+      UI_KEY_GEOSEMIMINOR,
+      0,
+      0,
+      "Geometric RMS along minor axis (ignoring value).",
+      UI_GROUP_COLUMNS_ELLIPTICAL,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "geo-axis-ratio",
+      UI_KEY_GEOAXISRATIO,
+      0,
+      0,
+      "Geometric (ignoring values, only lab) axis ratio.",
+      UI_GROUP_COLUMNS_ELLIPTICAL,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "geo-position-angle",
+      UI_KEY_GEOPOSITIONANGLE,
+      0,
+      0,
+      "Geometric (ignoring values, only lab) pos. angle.",
+      UI_GROUP_COLUMNS_ELLIPTICAL,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+
+
+
+
+
+    /* Multi-valued measurements. */
+    {
+      0, 0, 0, 0,
+      "Vector (multi-valued) measurements",
+      UI_GROUP_COLUMNS_VECTOR
+    },
+    {
+      "sum-in-slice",
+      UI_KEY_SUMINSLICE,
+      0,
+      0,
+      "[3D input] Sum of values in each slice.",
+      UI_GROUP_COLUMNS_VECTOR,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "sum-err-in-slice",
+      UI_KEY_SUMERRINSLICE,
+      0,
+      0,
+      "[3D input] Error in '--sum-in-slice'.",
+      UI_GROUP_COLUMNS_VECTOR,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "area-in-slice",
+      UI_KEY_AREAINSLICE,
+      0,
+      0,
+      "[3D input] Number of labeled in each slice.",
+      UI_GROUP_COLUMNS_VECTOR,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "sum-proj-in-slice",
+      UI_KEY_SUMPROJINSLICE,
+      0,
+      0,
+      "[3D input] Sum of projected area in each slice.",
+      UI_GROUP_COLUMNS_VECTOR,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "area-proj-in-slice",
+      UI_KEY_AREAPROJINSLICE,
+      0,
+      0,
+      "[3D input] Num. voxels in '--sum-proj-in-slice'.",
+      UI_GROUP_COLUMNS_VECTOR,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "sum-proj-err-in-slice",
+      UI_KEY_SUMPROJERRINSLICE,
+      0,
+      0,
+      "[3D input] Error of '--sum-proj-in-slice'.",
+      UI_GROUP_COLUMNS_VECTOR,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "area-other-in-slice",
+      UI_KEY_AREAOTHERINSLICE,
+      0,
+      0,
+      "[3D input] Area of other lab. in projected area.",
+      UI_GROUP_COLUMNS_VECTOR,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "sum-other-in-slice",
+      UI_KEY_SUMOTHERINSLICE,
+      0,
+      0,
+      "[3D input] Sum of other lab. in projected area.",
+      UI_GROUP_COLUMNS_VECTOR,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "sum-other-err-in-slice",
+      UI_KEY_SUMOTHERERRINSLICE,
+      0,
+      0,
+      "[3D input] Area in '--sum-other-in-slice'.",
+      UI_GROUP_COLUMNS_VECTOR,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+
+
+
+
     {0}
   };
 
