@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+Copyright (C) 2016-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -227,15 +227,14 @@ ui_column_codes_ll(struct argp_option *option, char *arg,
       if( arg[0]=='0' && arg[1]=='\0' ) return NULL;
       else if ( !(arg[0]=='1' && arg[1]=='\0')  )
         error_at_line(EXIT_FAILURE, 0, filename, lineno, "'%s' is not a "
-                      "valid value to the '%s' option: ('%s').\n\n'%s' is "
-                      "an on/off option specifying if you want this column "
-                      "in the output catalog, it doesn't take any "
-                      "arguments. In a configuration file, it can only take "
-                      "a value of '0' (to be ignored) or '1'", arg,
+                      "valid value to the '%s' option: ('%s').\n\n'%s' "
+                      "is an on/off option specifying if you want this "
+                      "column in the output catalog, it doesn't take any "
+                      "arguments. In a configuration file, it can only "
+                      "take a value of '0' (to be ignored) or '1'", arg,
                       option->name, option->doc, option->name);
     }
 
-
   /* The user wants this column, so add it to the list. Note that the 'ids'
      column means three columns. */
   if(option->key==UI_KEY_IDS)
@@ -287,7 +286,8 @@ ui_check_upperlimit(struct argp_option *option, char *arg,
       if(option->set) return NULL;
 
       /* Read the list of numbers as an array. */
-      raw=gal_options_parse_list_of_numbers(arg, filename, lineno);
+      raw=gal_options_parse_list_of_numbers(arg, filename, lineno,
+                                            GAL_TYPE_FLOAT64);
 
       /* Make sure there is at most only two numbers given. */
       if(raw->size>2)
@@ -295,16 +295,18 @@ ui_check_upperlimit(struct argp_option *option, char *arg,
                       "'--%s') contains %zu numbers, but only one or two "
                       "are acceptable.\n\n"
                       "With this option MakeCatalog will write all the "
-                      "positions and values of the random distribution for "
-                      "one particular labeled region into a table. The "
-                      "given value(s) is(are) the label identifier.\n\n"
+                      "positions and values of the random distribution "
+                      "for one particular labeled region into a table. "
+                      "The given value(s) is(are) the label "
+                      "identifier.\n\n"
                       "With one value the distribution for an object will "
-                      "be printed: the givne number will be interpretted as "
-                      "the requested object's label. With two values, the "
-                      "distribution for a specific clump will be written. "
-                      "The first will be interpretted as the clump's host "
-                      "object label and the second as the clump's label "
-                      "within the object", arg, option->name, raw->size);
+                      "be printed: the givne number will be interpretted "
+                      "as the requested object's label. With two values, "
+                      "the distribution for a specific clump will be "
+                      "written. The first will be interpretted as the "
+                      "clump's host object label and the second as the "
+                      "clump's label within the object", arg,
+                      option->name, raw->size);
 
       /* Make sure the given values are integers and that they are larger
          than zero. */
@@ -315,8 +317,8 @@ ui_check_upperlimit(struct argp_option *option, char *arg,
             error_at_line(EXIT_FAILURE, 0, filename, lineno, "%g (value "
                           "number %zu given to '--%s') is not an "
                           "integer. The value(s) to this option are "
-                          "object/clump labels/identifiers, so they must be "
-                          "integers", d[i], i+1, option->name);
+                          "object/clump labels/identifiers, so they must "
+                          "be integers", d[i], i+1, option->name);
           if( d[i]<=0 )
             error_at_line(EXIT_FAILURE, 0, filename, lineno, "%g (value "
                           "number %zu given to '--%s') is not positive. "
@@ -375,10 +377,10 @@ ui_read_check_only_options(struct mkcatalogparams *p)
     error(EXIT_FAILURE, 0, "no clumps catalog is requested, hence "
           "'--checkuplim' is only available for objects (one value "
           "must be given to it).\n\n"
-          "To ask for a clumps catalog, please append '--clumpscat' to the "
-          "command calling MakeCatalog.\n\n"
-          "If you want the upperlimit check table for an object, only give "
-          "one value (the object's label) to '--checkuplim'.");
+          "To ask for a clumps catalog, please append '--clumpscat' to "
+          "the command calling MakeCatalog.\n\n"
+          "If you want the upperlimit check table for an object, only "
+          "give one value (the object's label) to '--checkuplim'.");
 
   /* See if '--skyin' is a filename or a value. When the string is ONLY a
      number (and nothing else), 'tailptr' will point to the end of the
@@ -389,8 +391,8 @@ ui_read_check_only_options(struct mkcatalogparams *p)
       if(*tailptr=='\0')
         {
           /* Allocate the data structure. */
-          p->sky=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL, 0, -1,
-                                1, NULL, NULL, NULL);
+          p->sky=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL,
+                                0, -1, 1, NULL, NULL, NULL);
 
           /* Write the value inside it. */
           *((float *)(p->sky->array))=tmp;
@@ -404,8 +406,8 @@ ui_read_check_only_options(struct mkcatalogparams *p)
       if(*tailptr=='\0')
         {
           /* Allocate the data structure. */
-          p->std=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL, 0, -1,
-                                1, NULL, NULL, NULL);
+          p->std=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &one, NULL,
+                                0, -1, 1, NULL, NULL, NULL);
 
           /* Write the value inside it. */
           *((float *)(p->std->array))=tmp;
@@ -415,7 +417,7 @@ ui_read_check_only_options(struct mkcatalogparams *p)
   /* Make sure that '--fracmax' is given if necessary and that the fracsum
      values are less than one. */
   for(colcode=p->columnids; colcode!=NULL; colcode=colcode->next)
-    if( (colcode->v==UI_KEY_FRACMAXAREA1 || colcode->v==UI_KEY_FRACMAXAREA2)
+    if( (colcode->v==UI_KEY_FRACMAX1AREA || colcode->v==UI_KEY_FRACMAX2AREA)
         && p->fracmax==NULL )
       error(EXIT_FAILURE, 0, "please specify your required fractions for "
             "'--fracmaxarea' using the '--fracmax' option (for example "
@@ -437,7 +439,7 @@ ui_read_check_only_options(struct mkcatalogparams *p)
       /* If a second fracum column is necessary, make sure two values are
          given to --fracmax. */
       for(colcode=p->columnids; colcode!=NULL; colcode=colcode->next)
-        if(colcode->v==UI_KEY_FRACMAXAREA2 && p->fracmax->size==1)
+        if(colcode->v==UI_KEY_FRACMAX2AREA && p->fracmax->size==1)
           error(EXIT_FAILURE, 0, "only one value given to '--fracmax', "
                 "but '--fracmaxarea2' was requested! You need to give "
                 "the requested fraction as a second value to '--fracmax', "
@@ -456,10 +458,11 @@ ui_check_options_and_arguments(struct mkcatalogparams *p)
   if(p->objectsfile)
     {
       if( gal_fits_file_recognized(p->objectsfile) && p->cp.hdu==NULL )
-        error(EXIT_FAILURE, 0, "no HDU specified. When the input is a FITS "
-              "file, a HDU must also be specified, you can use the '--hdu' "
-              "('-h') option and give it the HDU number (starting from "
-              "zero), extension name, or anything acceptable by CFITSIO");
+        error(EXIT_FAILURE, 0, "no HDU specified. When the input is a "
+              "FITS file, a HDU must also be specified, you can use the "
+              "'--hdu' ('-h') option and give it the HDU number (starting "
+              "from zero), extension name, or anything acceptable by "
+              "CFITSIO");
 
     }
   else
@@ -497,13 +500,13 @@ ui_set_filenames(struct mkcatalogparams *p)
 
   p->usedvaluesfile = p->valuesfile ? p->valuesfile : p->objectsfile;
 
-  p->usedskyfile    = ( p->skyfile
-                       ? p->skyfile
-                       : ( p->valuesfile ? p->valuesfile : p->objectsfile ) );
+  p->usedskyfile = ( p->skyfile
+                     ? p->skyfile
+                     : (p->valuesfile ? p->valuesfile : p->objectsfile) );
 
-  p->usedstdfile    = ( p->stdfile
-                       ? p->stdfile
-                       : ( p->valuesfile ? p->valuesfile : p->objectsfile ) );
+  p->usedstdfile = ( p->stdfile
+                     ? p->stdfile
+                     : (p->valuesfile ? p->valuesfile : p->objectsfile) );
 }
 
 
@@ -557,10 +560,11 @@ ui_wcs_info(struct mkcatalogparams *p)
       for(i=0;i<p->objects->ndim;++i)
         {
           /* CTYPE might contain '-' characters, we just want the first
-             non-dash characters. The loop will either stop either at the end
-             or where there is a dash. So we can just replace it with an
-             end-of-string character. */
-          gal_checkset_allocate_copy(p->objects->wcs->ctype[i], &p->ctype[i]);
+             non-dash characters. The loop will either stop either at the
+             end or where there is a dash. So we can just replace it with
+             an end-of-string character. */
+          gal_checkset_allocate_copy(p->objects->wcs->ctype[i],
+                                     &p->ctype[i]);
           c=p->ctype[i]; while(*c!='\0' && *c!='-') ++c;
           *c='\0';
         }
@@ -581,7 +585,8 @@ ui_num_clumps(struct mkcatalogparams *p)
   gal_list_i32_t *tmp, **labsinobj;
   int32_t *o=p->objects->array, *of=o+p->objects->size, *c=p->clumps->array;
 
-  /* Allocate array of lists to keep the unique labels within each object. */
+  /* Allocate array of lists to keep the unique labels within each
+     object. */
   errno=0;
   labsinobj=calloc(p->numobjects+1, sizeof *labsinobj);
   if(labsinobj==NULL)
@@ -597,7 +602,8 @@ ui_num_clumps(struct mkcatalogparams *p)
         {
           /* See if the label has already been found. */
           olab = p->outlabsinv ? p->outlabsinv[*o] : *o;
-          for(tmp=labsinobj[olab];tmp!=NULL;tmp=tmp->next) if(tmp->v==*c) break;
+          for(tmp=labsinobj[olab];tmp!=NULL;tmp=tmp->next)
+            if(tmp->v==*c) break;
 
           /* When it wasn't found, 'tmp==NULL'. */
           if(tmp==NULL)
@@ -686,23 +692,42 @@ ui_one_tile_per_object_correct_numobjects(struct mkcatalogparams *p)
   start=p->objects->array;
   lf=(l=p->objects->array)+p->objects->size;
   do
-    if(*l>0)
-      {
-        /* Get the coordinates of this pixel. */
-        gal_dimension_index_to_coord(l-start, ndim, p->objects->dsize,
-                                     coord);
+    {
+      /* Small sanity check: the objects image shouldn't have negative
+         values (can happen when the user gives a clump image). */
+      if(*l!=GAL_BLANK_INT32 && *l<0)
+        error(EXIT_FAILURE, 0, "the main labeled image ('OBJECTS' HDU "
+              "when using Segment), shouldn't contain negative values. "
+              "This can happen when you mistakenly give Segment's "
+              "clumps as the main input labeled image. If you want to "
+              "treat clumps as objects, please use 'astarithmetic' to "
+              "convert all negative pixels to 0 before calling "
+              "MakeCatalog. For example this command: "
+              "'astarithmetic file.fits -hCLUMPS set-i i i 0 lt "
+              "0 where'. For this scenario, a better solution is to "
+              "only keep the clumps and give them each a separate "
+              "label with a command like this: 'astarithmetic file.fits "
+              "-hCLUMPS set-i i i 0 gt 2 connected-components'");
 
-        /* Check to see this coordinate is the smallest/largest found so
-           far for this label. Note that labels start from 1, while indexs
-           here start from zero. */
-        min = &minmax[ (*l-1) * width        ];
-        max = &minmax[ (*l-1) * width + ndim ];
-        for(d=0;d<ndim;++d)
-          {
-            if( coord[d] < min[d] ) min[d] = coord[d];
-            if( coord[d] > max[d] ) max[d] = coord[d];
-          }
-      }
+      /* We are on an object. */
+      if(*l>0)
+        {
+          /* Get the coordinates of this pixel. */
+          gal_dimension_index_to_coord(l-start, ndim, p->objects->dsize,
+                                       coord);
+
+          /* Check to see this coordinate is the smallest/largest found so
+             far for this label. Note that labels start from 1, while indexs
+             here start from zero. */
+          min = &minmax[ (*l-1) * width        ];
+          max = &minmax[ (*l-1) * width + ndim ];
+          for(d=0;d<ndim;++d)
+            {
+              if( coord[d] < min[d] ) min[d] = coord[d];
+              if( coord[d] > max[d] ) max[d] = coord[d];
+            }
+        }
+    }
   while(++l<lf);
 
   /* If a label doesn't exist in the image, then write over it and define
@@ -813,6 +838,7 @@ ui_one_tile_per_object_correct_numobjects(struct mkcatalogparams *p)
 static void
 ui_read_labels(struct mkcatalogparams *p)
 {
+  gal_list_i32_t *colcode;
   gal_data_t *tmp, *keys=gal_data_array_calloc(2);
 
   /* Read it into memory. */
@@ -836,11 +862,25 @@ ui_read_labels(struct mkcatalogparams *p)
           "currently only supports 2D or 3D datasets", p->objectsfile,
           p->cp.hdu, p->objects->ndim);
 
-  /* Make sure the '--spectrum' option is not given on a 2D image.  */
-  if(p->spectrum && p->objects->ndim!=3)
-    error(EXIT_FAILURE, 0, "%s (hdu %s) has %zu dimensions, but '--spectrum' "
-          "is currently only defined on 3D datasets", p->objectsfile,
-          p->cp.hdu, p->objects->ndim);
+
+  /* If a column needs a 3D input, do the check here.  */
+  for(colcode=p->columnids; colcode!=NULL; colcode=colcode->next)
+    switch(colcode->v)
+      {
+      case UI_KEY_SUMINSLICE:         case UI_KEY_SUMERRINSLICE:
+      case UI_KEY_AREAINSLICE:        case UI_KEY_SUMPROJINSLICE:
+      case UI_KEY_AREAPROJINSLICE:    case UI_KEY_SUMPROJERRINSLICE:
+      case UI_KEY_AREAOTHERINSLICE:   case UI_KEY_SUMOTHERINSLICE:
+      case UI_KEY_SUMOTHERERRINSLICE:
+        if(p->objects->ndim!=3)
+          error(EXIT_FAILURE, 0, "%s (hdu %s) has %zu dimensions, but "
+                "at least one requested column requires a 3D input "
+                "(for example those ending with 'inslice', they are "
+                "clearly marked with '[3D input]' in the output of "
+                "'%s --help')", p->objectsfile, p->cp.hdu,
+                p->objects->ndim, PROGRAM_EXEC);
+        break;
+      }
 
 
   /* See if the total number of objects is given in the header keywords. */
@@ -851,7 +891,7 @@ ui_read_labels(struct mkcatalogparams *p)
   if(keys[0].status) /* status!=0: the key couldn't be read by CFITSIO. */
     {
       tmp=gal_statistics_maximum(p->objects);
-      p->numobjects=*((int32_t *)(tmp->array)); /*numobjects is in int32_t.*/
+      p->numobjects=*((int32_t *)(tmp->array)); /*numobjects is int32_t.*/
       gal_data_free(tmp);
     }
 
@@ -860,8 +900,8 @@ ui_read_labels(struct mkcatalogparams *p)
      error (it is pointless to build a catalog). */
   if(p->numobjects==0)
     error(EXIT_FAILURE, 0, "no object labels (non-zero pixels) in "
-          "%s (hdu %s). To make a catalog, labeled regions must be defined",
-          p->objectsfile, p->cp.hdu);
+          "%s (hdu %s). To make a catalog, labeled regions must be "
+          "defined", p->objectsfile, p->cp.hdu);
 
 
   /* See if the labels image has blank pixels and set the flags
@@ -888,8 +928,8 @@ ui_read_labels(struct mkcatalogparams *p)
               "give a specific HDU using its number (counting from zero) "
               "or name. If the dataset is in another file, please use "
               "'--clumpsfile' to give the filename. If you don't want any "
-              "clumps catalog output, remove the '--clumpscat' option from "
-              "the command-line or give it a value of zero in a "
+              "clumps catalog output, remove the '--clumpscat' option "
+              "from the command-line or give it a value of zero in a "
               "configuration file", p->usedclumpsfile);
 
       /* Read the clumps image. */
@@ -932,11 +972,11 @@ ui_read_labels(struct mkcatalogparams *p)
             error(EXIT_FAILURE, 0, "%s (hdu: %s): the 'NUMCLUMPS' header "
                   "keyword has a value of zero, but there are positive "
                   "pixels in the array, showing that there are clumps in "
-                  "image. This is a wrong usage of the 'NUMCLUMPS' keyword."
-                  "It must contain the total number of clumps (irrespective "
-                  "of how many objects there are). Please correct this issue "
-                  "and run MakeCatalog again", p->usedclumpsfile,
-                  p->clumpshdu);
+                  "image. This is a wrong usage of the 'NUMCLUMPS' "
+                  "keyword. It must contain the total number of clumps "
+                  "(irrespective of how many objects there are). Please "
+                  "correct this issue and run MakeCatalog again",
+                  p->usedclumpsfile, p->clumpshdu);
 
           /* Since there are no clumps, we won't bother creating a clumps
              catalog and from this step onward, we'll act as if no clumps
@@ -974,7 +1014,6 @@ ui_necessary_inputs(struct mkcatalogparams *p, int *values, int *sky,
   /* Set necessary inputs based on options. */
   if(p->forcereadstd) *std=1;
   if(p->upperlimit) *values=1;
-  if(p->spectrum) *values=*std=1;
 
   /* Go over all the object columns. Note that the objects and clumps (if
      the '--clumpcat' option is given) inputs are mandatory and it is not
@@ -988,7 +1027,9 @@ ui_necessary_inputs(struct mkcatalogparams *p, int *values, int *sky,
         case OCOL_NUM:                *values        = 1;          break;
         case OCOL_NUMXY:              *values        = 1;          break;
         case OCOL_SUM:                *values        = 1;          break;
+        case OCOL_SUMP2:              *values        = 1;          break;
         case OCOL_SUM_VAR:            *values = *std = 1;          break;
+        case OCOL_SUM_VAR_NUM:        *values = *std = 1;          break;
         case OCOL_MEDIAN:             *values        = 1;          break;
         case OCOL_MAXIMUM:            *values        = 1;          break;
         case OCOL_SIGCLIPNUM:         *values        = 1;          break;
@@ -1041,6 +1082,17 @@ ui_necessary_inputs(struct mkcatalogparams *p, int *values, int *sky,
         case OCOL_C_GZ:               /* Only clump labels. */     break;
         case OCOL_C_SUMWHT:           *values        = 1;          break;
         case OCOL_C_NUMWHT:           *values        = 1;          break;
+        case OCOL_SUMINSLICE:         *values        = 1;          break;
+        case OCOL_NUMINSLICE:         *values        = 1;          break;
+        case OCOL_NUMALLINSLICE:      /* Only object labels. */    break;
+        case OCOL_SUMVARINSLICE:      *values = *std = 1;          break;
+        case OCOL_SUMPROJINSLICE:     *values        = 1;          break;
+        case OCOL_NUMPROJINSLICE:     *values        = 1;          break;
+        case OCOL_NUMOTHERINSLICE:    *values        = 1;          break;
+        case OCOL_SUMOTHERINSLICE:    *values        = 1;          break;
+        case OCOL_SUMPROJVARINSLICE:  *values = *std = 1;          break;
+        case OCOL_SUMOTHERVARINSLICE: *values = *std = 1;          break;
+        case OCOL_NUMALLOTHERINSLICE: /* Only object labels. */    break;
         default:
           error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to "
                 "fix the problem. The code %zu is not a recognized "
@@ -1060,6 +1112,7 @@ ui_necessary_inputs(struct mkcatalogparams *p, int *values, int *sky,
           case CCOL_NUMXY:            *values        = 1;          break;
           case CCOL_SUM:              *values        = 1;          break;
           case CCOL_SUM_VAR:          *values = *std = 1;          break;
+          case CCOL_SUM_VAR_NUM:      *values = *std = 1;          break;
           case CCOL_MEDIAN:           *values        = 1;          break;
           case CCOL_MAXIMUM:          *values        = 1;          break;
           case CCOL_SIGCLIPNUM:       *values        = 1;          break;
@@ -1241,7 +1294,8 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
               "give the filename", p->usedvaluesfile);
 
       /* Read the values dataset. */
-      p->values=gal_array_read_one_ch_to_type(p->usedvaluesfile, p->valueshdu,
+      p->values=gal_array_read_one_ch_to_type(p->usedvaluesfile,
+                                              p->valueshdu,
                                               NULL, GAL_TYPE_FLOAT32,
                                               p->cp.minmapsize,
                                               p->cp.quietmmap);
@@ -1265,12 +1319,16 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
         {
           for(column=p->objectcols; column!=NULL; column=column->next)
             if( !strcmp(column->unit, MKCATALOG_NO_UNIT) )
-              { free(column->unit);
-                gal_checkset_allocate_copy(p->values->unit, &column->unit); }
+              {
+                free(column->unit);
+                gal_checkset_allocate_copy(p->values->unit, &column->unit);
+              }
           for(column=p->clumpcols; column!=NULL; column=column->next)
             if( !strcmp(column->unit, MKCATALOG_NO_UNIT) )
-              { free(column->unit);
-                gal_checkset_allocate_copy(p->values->unit, &column->unit); }
+              {
+                free(column->unit);
+                gal_checkset_allocate_copy(p->values->unit, &column->unit);
+              }
         }
     }
 
@@ -1287,10 +1345,10 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
             error(EXIT_FAILURE, 0, "%s: no HDU/extension provided for the "
                   "SKY dataset. Atleast one column needs this dataset, or "
                   "you have asked to subtract the Sky from the values.\n\n"
-                  "Please use the '--skyhdu' option to give a specific HDU "
-                  "using its number (counting from zero) or name. If the "
-                  "dataset is in another file, please use '--skyin' to "
-                  "give the filename", p->usedskyfile);
+                  "Please use the '--skyhdu' option to give a specific "
+                  "HDU using its number (counting from zero) or name. If "
+                  "the dataset is in another file, please use '--skyin' "
+                  "to give the filename", p->usedskyfile);
 
           /* Read the Sky dataset. */
           p->sky=gal_array_read_one_ch_to_type(p->usedskyfile, p->skyhdu,
@@ -1327,7 +1385,8 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
       /* Read the Sky standard deviation image into memory. */
       p->std=gal_array_read_one_ch_to_type(p->usedstdfile, p->stdhdu,
                                            NULL, GAL_TYPE_FLOAT32,
-                                           p->cp.minmapsize, p->cp.quietmmap);
+                                           p->cp.minmapsize,
+                                           p->cp.quietmmap);
       p->std->ndim=gal_dimension_remove_extra(p->std->ndim,
                                               p->std->dsize, NULL);
 
@@ -1370,17 +1429,17 @@ ui_preparations_read_inputs(struct mkcatalogparams *p)
 
           /* Check its size. */
           if( gal_dimension_is_different(p->objects, p->upmask) )
-            error(EXIT_FAILURE, 0, "'%s' (hdu: %s) and '%s' (hdu: %s) have a"
-                  "different dimension/size", p->upmaskfile, p->upmaskhdu,
-                  p->objectsfile, p->cp.hdu);
+            error(EXIT_FAILURE, 0, "'%s' (hdu: %s) and '%s' (hdu: %s) "
+                  "have a different dimension/size", p->upmaskfile,
+                  p->upmaskhdu, p->objectsfile, p->cp.hdu);
 
           /* If it isn't an integer type, report an error. */
           if( p->upmask->type==GAL_TYPE_FLOAT32
               || p->upmask->type==GAL_TYPE_FLOAT64 )
             error(EXIT_FAILURE, 0, "%s (hdu: %s) has a %s numerical data "
-                  "type. Only integer type inputs are acceptable as a mask."
-                  "If the values are indeed integers, only placed in a "
-                  "floating point container, you can use Gnuastro's "
+                  "type. Only integer type inputs are acceptable as a "
+                  "mask. If the values are indeed integers, only placed "
+                  "in a floating point container, you can use Gnuastro's "
                   "Arithmetic program to conver the numeric data type",
                   p->upmaskfile, p->upmaskhdu,
                   gal_type_name(p->upmask->type, 1));
@@ -1418,9 +1477,9 @@ ui_preparations_read_keywords(struct mkcatalogparams *p)
           /* Read the keywords from the standard deviation image. */
           keys=gal_data_array_calloc(2);
           keys[0].next=&keys[1];
-          keys[0].name="MINSTD";              keys[1].name="MEDSTD";
-          keys[0].type=GAL_TYPE_FLOAT32;      keys[1].type=GAL_TYPE_FLOAT32;
-          keys[0].array=&minstd;              keys[1].array=&p->medstd;
+          keys[0].name="MINSTD";          keys[1].name="MEDSTD";
+          keys[0].type=GAL_TYPE_FLOAT32;  keys[1].type=GAL_TYPE_FLOAT32;
+          keys[0].array=&minstd;          keys[1].array=&p->medstd;
           gal_fits_key_read(p->usedstdfile, p->stdhdu, keys, 0, 0);
 
           /* If the two keywords couldn't be read. We don't want to slow
@@ -1513,12 +1572,26 @@ ui_preparations_both_names(struct mkcatalogparams *p)
      file. */
   if(p->cp.tableformat==GAL_TABLE_FORMAT_TXT)
     {
-      p->objectsout=gal_checkset_automatic_output(&p->cp, basename, "_o.txt");
-      p->clumpsout=gal_checkset_automatic_output(&p->cp, basename, "_c.txt");
+      p->objectsout=gal_checkset_automatic_output(&p->cp, basename,
+                                                  "_o.txt");
+      p->clumpsout=gal_checkset_automatic_output(&p->cp, basename,
+                                                 "_c.txt");
     }
   else
     {
-      p->objectsout=gal_checkset_automatic_output(&p->cp, basename, suffix);
+      /* The output file is a FITS file (the 'p->cp.tableformat' has been
+         set before), so a single file output is enough. If no output name
+         has been given, then use automatic output. Otherwise, just check
+         if the given name is writable. */
+      if(p->cp.output)
+        {
+          gal_checkset_allocate_copy(p->cp.output, &p->objectsout);
+          gal_checkset_writable_remove(p->objectsout, p->objectsfile, 0,
+                                       p->cp.dontdelete);
+        }
+      else
+        p->objectsout=gal_checkset_automatic_output(&p->cp, basename,
+                                                    suffix);
       p->clumpsout=p->objectsout;
     }
 
@@ -1555,7 +1628,8 @@ ui_preparations_outnames(struct mkcatalogparams *p)
       if(p->clumps) ui_preparations_both_names(p);
       else
         {
-          gal_checkset_writable_remove(p->cp.output, 0, p->cp.dontdelete);
+          gal_checkset_writable_remove(p->cp.output, p->objectsfile, 0,
+                                       p->cp.dontdelete);
           gal_checkset_allocate_copy(p->cp.output, &p->objectsout);
         }
     }
@@ -1569,7 +1643,8 @@ ui_preparations_outnames(struct mkcatalogparams *p)
         {
           suffix = ( p->cp.tableformat==GAL_TABLE_FORMAT_TXT
                      ? "_cat.txt" : "_cat.fits" );
-          p->objectsout=gal_checkset_automatic_output(&p->cp, p->objectsfile,
+          p->objectsout=gal_checkset_automatic_output(&p->cp,
+                                                      p->objectsfile,
                                                       suffix);
         }
     }
@@ -1605,91 +1680,6 @@ ui_preparations_outnames(struct mkcatalogparams *p)
 
 
 
-/* When a spectrum is requested, the slice information (slice number and
-   slice WCS) is common to all different spectra. So instead of calculating
-   it every time, we'll just make it once here, then copy it for every
-   object.
-
-   The Slice information is going to be written in every spectrum. So we
-   don't want it to take too much space. Therefore, only when the number of
-   slices is less than 65535 (2^16-1), will we actually use a 32-bit
-   integer type for the slice number column.
-*/
-static void
-ui_preparations_spectrum_wcs(struct mkcatalogparams *p)
-{
-  double *xarr, *yarr, *zarr;
-  gal_data_t *x, *y, *z, *coords;
-  size_t i, numslices=p->objects->dsize[0];
-  size_t slicenumtype=numslices>=65535 ? GAL_TYPE_UINT32 : GAL_TYPE_UINT16;
-
-  /* A small sanity check. */
-  if(p->objects->ndim!=3)
-    error(EXIT_FAILURE, 0, "%s (hdu %s) is a %zuD dataset, but '--spectrum' "
-          "is currently only defined on 3D datasets", p->objectsfile,
-          p->cp.hdu, p->objects->ndim);
-
-  /* Allocate space for the slice number as well as the X and Y positions
-     for WCS conversion. Note that the 'z' axis is going to be converted to
-     WCS later, so we'll just give it the basic information now.*/
-  x=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &numslices, NULL, 0,
-                   p->cp.minmapsize, p->cp.quietmmap, NULL, NULL, NULL);
-  y=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &numslices, NULL, 0,
-                   p->cp.minmapsize, p->cp.quietmmap, NULL, NULL, NULL);
-  z=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &numslices, NULL, 0,
-                   p->cp.minmapsize, p->cp.quietmmap, p->ctype[2],
-                   p->objects->wcs->cunit[2], "Slice WCS coordinates.");
-
-  /* Write values into the 3 coordinates. */
-  xarr=x->array; yarr=y->array; zarr=z->array;
-  for(i=0;i<numslices;++i) { zarr[i]=i+1; xarr[i]=yarr[i]=1; }
-
-
-  /* Convert the coordinates to WCS. We are doing this inplace to avoid too
-     much memory/speed consumption. */
-  coords=x;
-  coords->next=y;
-  coords->next->next=z;
-  gal_wcs_img_to_world(coords, p->objects->wcs, 1);
-
-  /* For a check.
-  for(i=0;i<numslices;++i)
-    printf("%g, %g, %g\n", xarr[i], yarr[i], zarr[i]);
-  exit(0);
-  */
-
-  /* Allocate the slice counter array (we are doing it again because we
-     want it to be in integer type now). */
-  p->specsliceinfo=gal_data_alloc(NULL, slicenumtype, 1, &numslices, NULL, 0,
-                                  p->cp.minmapsize, p->cp.quietmmap, "SLICE",
-                                  "counter",
-                                  "Slice number in cube (counting from 1).");
-  if(p->specsliceinfo->type==GAL_TYPE_UINT16)
-    for(i=0;i<numslices;++i) ((uint16_t *)(p->specsliceinfo->array))[i]=i+1;
-  else
-    for(i=0;i<numslices;++i) ((uint32_t *)(p->specsliceinfo->array))[i]=i+1;
-
-  /* Set the slice WCS column information. Note that 'z' is now the WCS
-     coordinate value of the third dimension, and to avoid wasting extra
-     space (this column is repeated one very object's spectrum), we'll
-     convert it to a 32-bit floating point dataset. */
-  p->specsliceinfo->next=gal_data_copy_to_new_type(z, GAL_TYPE_FLOAT32);
-
-  /* For a final check.
-  gal_table_write(p->specsliceinfo, NULL, NULL, GAL_TABLE_FORMAT_BFITS,
-                  "specsliceinfo.fits", "test-debug", 0);
-  */
-
-  /* Clean up. */
-  gal_data_free(x);
-  gal_data_free(y);
-  gal_data_free(z);
-}
-
-
-
-
-
 /* Sanity checks and preparations for the upper-limit magnitude. */
 static void
 ui_preparations_upperlimit(struct mkcatalogparams *p)
@@ -1702,15 +1692,17 @@ ui_preparations_upperlimit(struct mkcatalogparams *p)
     {
       for(i=0;p->uprange[i]!=-1;++i) ++c;
       if(c!=p->objects->ndim)
-        error(EXIT_FAILURE, 0, "%zu values given to '--uprange', but input "
-              "has %zu dimensions", c, p->objects->ndim);
+        error(EXIT_FAILURE, 0, "%zu values given to '--uprange', but "
+              "input has %zu dimensions", c, p->objects->ndim);
     }
 
   /* Check the number of random samples. */
-  if( p->upnum < MKCATALOG_UPPERLIMIT_MINIMUM_NUM )
-    error(EXIT_FAILURE, 0, "%zu not acceptable as '--upnum'. The minimum "
-          "acceptable number of random samples for the upper limit "
-          "magnitude is %d", p->upnum, MKCATALOG_UPPERLIMIT_MINIMUM_NUM);
+  if( p->upnum < MKCATALOG_UPPERLIMIT_MINIMUM_NUM && p->cp.quiet==0 )
+    error(EXIT_SUCCESS, 0, "WARNING: %zu (value given to '--upnum') will "
+          "produce unreliable upper-limit measurements. This warning will "
+          "be printed for '--upnum' values less than %d. You can suppress "
+          "this warning by adding '--quiet' to your command", p->upnum,
+          MKCATALOG_UPPERLIMIT_MINIMUM_NUM);
 
   /* Check if sigma-clipping parameters have been given. */
   if( isnan(p->upsigmaclip[0]) )
@@ -1746,7 +1738,7 @@ void
 ui_preparations(struct mkcatalogparams *p)
 {
   /* If no columns are requested, then inform the user. */
-  if(p->columnids==NULL && p->spectrum==0)
+  if(p->columnids==NULL)
     error(EXIT_FAILURE, 0, "no measurements requested! Please run again "
           "with '--help' for the possible list of measurements");
 
@@ -1776,14 +1768,6 @@ ui_preparations(struct mkcatalogparams *p)
   ui_preparations_outnames(p);
 
 
-  /* If a spectrum is requested, generate the two WCS columns. */
-  if(p->spectrum)
-    {
-      ui_preparations_spectrum_wcs(p);
-      p->spectra=gal_data_array_calloc(p->numobjects);
-    }
-
-
   /* Allocate the reference random number generator and seed values. It
      will be cloned once for every thread. If the user hasn't called
      'envseed', then we want it to be different for every run, so we need
@@ -1835,7 +1819,8 @@ ui_preparations(struct mkcatalogparams *p)
 /**************************************************************/
 
 void
-ui_read_check_inputs_setup(int argc, char *argv[], struct mkcatalogparams *p)
+ui_read_check_inputs_setup(int argc, char *argv[],
+                           struct mkcatalogparams *p)
 {
   char *tmp;
   struct gal_options_common_params *cp=&p->cp;
@@ -1920,7 +1905,8 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct mkcatalogparams *p)
           else
             printf("  - Sky: %s (hdu: %s)\n", p->usedskyfile, p->skyhdu);
           if(p->subtractsky)
-            printf("    - Sky has been subtracted from values internally.\n");
+            printf("    - Sky has been subtracted from values "
+                   "internally.\n");
         }
 
       if(p->std)
@@ -1970,7 +1956,7 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct mkcatalogparams *p)
 void
 ui_free_report(struct mkcatalogparams *p, struct timeval *t1)
 {
-  size_t d, i;
+  size_t d;
 
   /* The temporary arrays for WCS coordinates. */
   if(p->wcs_vo ) gal_list_data_free(p->wcs_vo);
@@ -2019,27 +2005,10 @@ ui_free_report(struct mkcatalogparams *p, struct timeval *t1)
   if(p->outlabs) free(p->outlabs);
   gal_list_data_free(p->clumpcols);
   gal_list_data_free(p->objectcols);
-  gal_list_data_free(p->specsliceinfo);
   if(p->outlabsinv) free(p->outlabsinv);
   if(p->upcheckout) free(p->upcheckout);
   gal_data_array_free(p->tiles, p->numobjects, 0);
 
-  /* Clean up the spectra. */
-  if(p->spectra)
-    {
-      /* Note that each element of the array is the first node in a list of
-         datasets. So we can't free the first one with
-         'gal_list_data_free', we'll delete all the nodes after it in the
-         loop. */
-      for(i=0;i<p->numobjects;++i)
-        {
-          gal_list_data_free( p->spectra[i].next );
-          p->spectra[i].next=NULL;
-          gal_data_free_contents(&p->spectra[i]);
-        }
-      gal_data_array_free(p->spectra, p->numobjects, 0);
-    }
-
   /* If the Sky or its STD image were given in tiles, then we defined a
      tile structure to deal with them. The initialization of the tile
      structure is checked with its 'ndim' element. */
