@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2022 Free Software Foundation, Inc.
+Copyright (C) 2016-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -39,7 +39,10 @@ enum program_args_groups
   UI_GROUP_COLUMNS_POSITION_PIXEL,
   UI_GROUP_COLUMNS_POSITION_WCS,
   UI_GROUP_COLUMNS_BRIGHTNESS,
+  UI_GROUP_COLUMNS_SURFACEBRIGHTNESS,
   UI_GROUP_COLUMNS_MORPHOLOGY,
+  UI_GROUP_COLUMNS_ELLIPTICAL,
+  UI_GROUP_COLUMNS_VECTOR,
 };
 
 
@@ -48,8 +51,8 @@ enum program_args_groups
 
 /* Available letters for short options:
 
-   f g k w x y z
-   E H J L O R W X Y
+   a b c f g k n u w x y z
+   A B E G H J L O Q R W X Y
 */
 enum option_keys_enum
 {
@@ -63,22 +66,12 @@ enum option_keys_enum
 
   UI_KEY_IDS             = 'i',         /* Catalog columns. */
   UI_KEY_HOSTOBJID       = 'j',
-  UI_KEY_NUMCLUMPS       = 'c',
-  UI_KEY_AREA            = 'a',
   UI_KEY_X               = 'x',
   UI_KEY_Y               = 'y',
   UI_KEY_Z               = 'z',
   UI_KEY_RA              = 'r',
   UI_KEY_DEC             = 'd',
-  UI_KEY_BRIGHTNESS      = 'b',
   UI_KEY_MAGNITUDE       = 'm',
-  UI_KEY_MAGNITUDEERR    = 'G',
-  UI_KEY_UPPERLIMITMAG   = 'u',
-  UI_KEY_SN              = 'n',
-  UI_KEY_SEMIMAJOR       = 'A',
-  UI_KEY_SEMIMINOR       = 'B',
-  UI_KEY_AXISRATIO       = 'Q',
-  UI_KEY_POSITIONANGLE   = 'p',
 
   /* Only with long version (start with a value 1000, the rest will be set
      automatically). */
@@ -109,10 +102,12 @@ enum option_keys_enum
 
   UI_KEY_OBJID,                         /* Catalog columns. */
   UI_KEY_IDINHOSTOBJ,
+  UI_KEY_AREA,
   UI_KEY_AREAARCSEC2,
-  UI_KEY_SURFACEBRIGHTNESS,
+  UI_KEY_SB,
   UI_KEY_SBERROR,
   UI_KEY_AREAXY,
+  UI_KEY_NUMCLUMPS,
   UI_KEY_CLUMPSAREA,
   UI_KEY_WEIGHTAREA,
   UI_KEY_GEOAREA,
@@ -120,14 +115,14 @@ enum option_keys_enum
   UI_KEY_GEOX,
   UI_KEY_GEOY,
   UI_KEY_GEOZ,
-  UI_KEY_MINVX,
-  UI_KEY_MAXVX,
-  UI_KEY_MINVY,
-  UI_KEY_MAXVY,
-  UI_KEY_MINVZ,
-  UI_KEY_MAXVZ,
-  UI_KEY_MINVNUM,
-  UI_KEY_MAXVNUM,
+  UI_KEY_MINVALX,
+  UI_KEY_MAXVALX,
+  UI_KEY_MINVALY,
+  UI_KEY_MAXVALY,
+  UI_KEY_MINVALZ,
+  UI_KEY_MAXVALZ,
+  UI_KEY_MINVALNUM,
+  UI_KEY_MAXVALNUM,
   UI_KEY_CLUMPSX,
   UI_KEY_CLUMPSY,
   UI_KEY_CLUMPSZ,
@@ -152,21 +147,25 @@ enum option_keys_enum
   UI_KEY_CLUMPSGEOW1,
   UI_KEY_CLUMPSGEOW2,
   UI_KEY_CLUMPSGEOW3,
-  UI_KEY_BRIGHTNESSERR,
-  UI_KEY_CLUMPSBRIGHTNESS,
-  UI_KEY_BRIGHTNESSNORIVER,
+  UI_KEY_SUM,
+  UI_KEY_SUMERROR,
+  UI_KEY_CLUMPSSUM,
+  UI_KEY_SUMNORIVER,
   UI_KEY_STD,
   UI_KEY_MEAN,
   UI_KEY_MEDIAN,
   UI_KEY_MAXIMUM,
+  UI_KEY_MAGNITUDEERROR,
+  UI_KEY_SN,
   UI_KEY_CLUMPSMAGNITUDE,
   UI_KEY_UPPERLIMIT,
   UI_KEY_UPPERLIMITSB,
+  UI_KEY_UPPERLIMITMAG,
   UI_KEY_UPPERLIMITONESIGMA,
   UI_KEY_UPPERLIMITSIGMA,
   UI_KEY_UPPERLIMITQUANTILE,
   UI_KEY_UPPERLIMITSKEW,
-  UI_KEY_RIVERAVE,
+  UI_KEY_RIVERMEAN,
   UI_KEY_RIVERNUM,
   UI_KEY_SKY,
   UI_KEY_SKYSTD,
@@ -174,6 +173,13 @@ enum option_keys_enum
   UI_KEY_SIGCLIPMEDIAN,
   UI_KEY_SIGCLIPMEAN,
   UI_KEY_SIGCLIPSTD,
+  UI_KEY_SIGCLIPMEANSB,
+  UI_KEY_SIGCLIPMEANSBDELTA,
+  UI_KEY_SIGCLIPSTDSB,
+  UI_KEY_SEMIMAJOR,
+  UI_KEY_SEMIMINOR,
+  UI_KEY_POSITIONANGLE,
+  UI_KEY_AXISRATIO,
   UI_KEY_GEOSEMIMAJOR,
   UI_KEY_GEOSEMIMINOR,
   UI_KEY_GEOAXISRATIO,
@@ -186,12 +192,23 @@ enum option_keys_enum
   UI_KEY_HALFSUMAREA,
   UI_KEY_HALFSUMSB,
   UI_KEY_HALFSUMRADIUS,
-  UI_KEY_FRACMAXSUM1,
-  UI_KEY_FRACMAXSUM2,
-  UI_KEY_FRACMAXAREA1,
-  UI_KEY_FRACMAXAREA2,
-  UI_KEY_FRACMAXRADIUS1,
-  UI_KEY_FRACMAXRADIUS2,
+  UI_KEY_FRACMAX1SUM,
+  UI_KEY_FRACMAX2SUM,
+  UI_KEY_FRACMAX1AREA,
+  UI_KEY_FRACMAX2AREA,
+  UI_KEY_FRACMAX1RADIUS,
+  UI_KEY_FRACMAX2RADIUS,
+
+  /* Vector columns */
+  UI_KEY_SUMINSLICE,
+  UI_KEY_SUMERRINSLICE,
+  UI_KEY_AREAINSLICE,
+  UI_KEY_SUMPROJINSLICE,
+  UI_KEY_AREAPROJINSLICE,
+  UI_KEY_SUMPROJERRINSLICE,
+  UI_KEY_AREAOTHERINSLICE,
+  UI_KEY_SUMOTHERINSLICE,
+  UI_KEY_SUMOTHERERRINSLICE,
 };
 
 
