@@ -5,7 +5,7 @@ MakeNoise is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2022 Free Software Foundation, Inc.
+Copyright (C) 2016-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -84,13 +84,13 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET
     },
     {
-      "bgisbrightness",
-      UI_KEY_BGISBRIGHTNESS,
+      "bgnotmag",
+      UI_KEY_BGNOTMAG,
       0,
       0,
       "Background is brightness, not magnitude.",
       GAL_OPTIONS_GROUP_OPERATING_MODE,
-      &p->bgisbrightness,
+      &p->bgnotmag,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
       GAL_OPTIONS_NOT_MANDATORY,
