@@ -5,7 +5,7 @@ MakeNoise is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -237,17 +237,17 @@ ui_read_check_only_options(struct mknoiseparams *p)
   if( !isnan(p->background) )
     {
       /* Make sure that the background can be interpretted properly. */
-      if( p->bgisbrightness==0 && isnan(p->zeropoint) )
+      if( p->bgnotmag==0 && isnan(p->zeropoint) )
         error(EXIT_FAILURE, 0, "missing background information. When the "
               "noise is identified by the background, a zeropoint magnitude "
               "is mandatory. Please use the '--zeropoint' option to specify "
               "a zeropoint magnitude. Alternatively, if your background value "
               "is brightness (which is in linear scale and doesn't need a "
-              "zeropoint), please use '--bgisbrightness'");
+              "zeropoint), please use '--bgnotmag'");
 
       /* If the background is in units of magnitudes, convert it to
          brightness. */
-      if( p->bgisbrightness==0 )
+      if( p->bgnotmag==0 )
         p->background = pow(10, (p->zeropoint-p->background)/2.5f);
 
       /* Make sure that the background is larger than 1 (where Poisson
@@ -326,7 +326,8 @@ ui_preparations(struct mknoiseparams *p)
 
   /* Set the output name: */
   if(p->cp.output)
-    gal_checkset_writable_remove(p->cp.output, 0, p->cp.dontdelete);
+    gal_checkset_writable_remove(p->cp.output, p->inputname, 0,
+                                 p->cp.dontdelete);
   else
     p->cp.output=gal_checkset_automatic_output(&p->cp, p->inputname,
                                                "_noised.fits");
