@@ -5,7 +5,7 @@ MakeNoise is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -43,7 +43,7 @@ enum option_keys_enum
   UI_KEY_BACKGROUND     = 'b',
   UI_KEY_ZEROPOINT      = 'z',
   UI_KEY_ENVSEED        = 'e',
-  UI_KEY_BGISBRIGHTNESS = 'B',
+  UI_KEY_BGNOTMAG       = 'B',
 
   /* Only with long version (start with a value 1000, the rest will be set
      automatically). */
