@@ -5,7 +5,7 @@ NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -36,7 +36,7 @@ struct argp_option program_options[] =
     {
       "kernel",
       UI_KEY_KERNEL,
-      "STR",
+      "FITS",
       0,
       "Filename of kernel to convolve with input",
       GAL_OPTIONS_GROUP_INPUT,
@@ -62,7 +62,7 @@ struct argp_option program_options[] =
     {
       "convolved",
       UI_KEY_CONVOLVED,
-      "STR",
+      "FITS",
       0,
       "Convolved image file to avoid convolution.",
       GAL_OPTIONS_GROUP_INPUT,
@@ -88,7 +88,7 @@ struct argp_option program_options[] =
     {
       "widekernel",
       UI_KEY_WIDEKERNEL,
-      "STR",
+      "FITS",
       0,
       "Filename of wider kernel for better qthresh",
       GAL_OPTIONS_GROUP_INPUT,
@@ -137,13 +137,13 @@ struct argp_option program_options[] =
 
     /* Output options. */
     {
-      "ignoreblankinsky",
-      UI_KEY_IGNOREBLANKINSKY,
+      "ignoreblankintiles",
+      UI_KEY_IGNOREBLANKINTILES,
       0,
       0,
-      "Don't write input's blanks in the Sky output.",
+      "Don't write input's blanks in tiled output.",
       GAL_OPTIONS_GROUP_OUTPUT,
-      &p->ignoreblankinsky,
+      &p->ignoreblankintiles,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
       GAL_OPTIONS_NOT_MANDATORY,
@@ -211,6 +211,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "outliernumngb",
+      UI_KEY_OUTLIERNUMNGB,
+      "INT",
+      0,
+      "Num neighboring tiles to look for outlier.",
+      UI_GROUP_DETECTION,
+      &p->outliernumngb,
+      GAL_TYPE_SIZE_T,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "outliersclip",
       UI_KEY_OUTLIERSCLIP,
@@ -295,7 +308,7 @@ struct argp_option program_options[] =
       UI_KEY_ERODENGB,
       "INT",
       0,
-      "4 or 8 connectivity in erosion.",
+      "Connectivity in erosion (number of ngbs).",
       UI_GROUP_DETECTION,
       &p->erodengb,
       GAL_TYPE_SIZE_T,
@@ -325,7 +338,7 @@ struct argp_option program_options[] =
       UI_GROUP_DETECTION,
       &p->opening,
       GAL_TYPE_SIZE_T,
-      GAL_OPTIONS_RANGE_GT_0,
+      GAL_OPTIONS_RANGE_GE_0,
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
@@ -334,7 +347,7 @@ struct argp_option program_options[] =
       UI_KEY_OPENINGNGB,
       "INT",
       0,
-      "4 or 8 connectivity in opening.",
+      "Connectivity in opening (number of ngbs).",
       UI_GROUP_DETECTION,
       &p->openingngb,
       GAL_TYPE_SIZE_T,
@@ -408,6 +421,32 @@ struct argp_option program_options[] =
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "dopening",
+      UI_KEY_DOPENING,
+      "INT",
+      0,
+      "Depth of opening after dthresh.",
+      UI_GROUP_DETECTION,
+      &p->dopening,
+      GAL_TYPE_SIZE_T,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "dopeningngb",
+      UI_KEY_DOPENINGNGB,
+      "INT",
+      0,
+      "4 or 8 connectivity for dthresh opening.",
+      UI_GROUP_DETECTION,
+      &p->dopeningngb,
+      GAL_TYPE_SIZE_T,
+      GAL_OPTIONS_RANGE_GT_0,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "holengb",
       UI_KEY_HOLENGB,
@@ -421,6 +460,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "pseudoconcomp",
+      UI_KEY_PSEUDOCONCOMP,
+      "INT",
+      0,
+      "4 or 8 neighbors for labeling pseudo-dets.",
+      UI_GROUP_DETECTION,
+      &p->pseudoconcomp,
+      GAL_TYPE_SIZE_T,
+      GAL_OPTIONS_RANGE_GT_0,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "snminarea",
       UI_KEY_SNMINAREA,
@@ -473,6 +525,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "snthresh",
+      UI_KEY_SNTHRESH,
+      "FLT",
+      0,
+      "Manually input pseudo-det S/N threshold.",
+      UI_GROUP_DETECTION,
+      &p->snthresh,
+      GAL_TYPE_FLOAT32,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "detgrowquant",
       UI_KEY_DETGROWQUANT,
