@@ -5,7 +5,7 @@ Statistics is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -640,7 +640,7 @@ statistics_output_name(struct statisticsparams *p, char *suf, int *isfits)
   *isfits=strcmp(fix, "fits")==0;
 
   /* Make sure it doesn't already exist. */
-  gal_checkset_writable_remove(out, 0, p->cp.dontdelete);
+  gal_checkset_writable_remove(out, p->inputname, 0, p->cp.dontdelete);
 
   /* Clean up and return. */
   if(suffix) free(suffix);
@@ -677,7 +677,7 @@ write_output_table(struct statisticsparams *p, gal_data_t *table,
 
 
   /* Write the table. */
-  gal_checkset_writable_remove(output, 0, p->cp.dontdelete);
+  gal_checkset_writable_remove(output, p->inputname, 0, p->cp.dontdelete);
   gal_table_write(table, NULL, comments, p->cp.tableformat, output,
                   "TABLE", 0);
 
