@@ -5,7 +5,7 @@ Table is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2022 Free Software Foundation, Inc.
+Copyright (C) 2016-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -42,7 +42,8 @@ struct argp_option program_options[] =
       GAL_TYPE_STRLL,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
+      GAL_OPTIONS_NOT_SET,
+      gal_options_parse_csv_strings_append
     },
     {
       "wcsfile",
@@ -168,32 +169,18 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET
     },
     {
-      "catcolumnrawname",
-      UI_KEY_CATCOLUMNRAWNAME,
+      "txteasy",
+      UI_KEY_TXTEASY,
       0,
       0,
-      "Don't touch column names of --catcolumnfile.",
+      "Short for '-Afixed -B6 -ffixed -p3'.",
       GAL_OPTIONS_GROUP_OUTPUT,
-      &p->catcolumnrawname,
+      &p->txteasy,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
-    {
-      "colmetadata",
-      UI_KEY_COLMETADATA,
-      "STR,STR[,STR,STR]",
-      0,
-      "Update output metadata (name, unit, comments).",
-      GAL_OPTIONS_GROUP_OUTPUT,
-      &p->colmetadata,
-      GAL_TYPE_STRING,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET,
-      gal_options_parse_name_and_strings
-    },
     {
       "txtf32format",
       UI_KEY_TXTF32FORMAT,
@@ -228,7 +215,7 @@ struct argp_option program_options[] =
       "Text output float32 precision.",
       GAL_OPTIONS_GROUP_OUTPUT,
       &p->txtf32precision,
-      GAL_TYPE_SIZE_T,
+      GAL_TYPE_INT,
       GAL_OPTIONS_RANGE_GE_0,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
@@ -238,10 +225,10 @@ struct argp_option program_options[] =
       UI_KEY_TXTF64PRECISION,
       "INT",
       0,
-      "Text output float32 precision.",
+      "Text output float64 precision.",
       GAL_OPTIONS_GROUP_OUTPUT,
       &p->txtf64precision,
-      GAL_TYPE_SIZE_T,
+      GAL_TYPE_INT,
       GAL_OPTIONS_RANGE_GE_0,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
@@ -251,6 +238,120 @@ struct argp_option program_options[] =
 
 
 
+    /* Operation precendence */
+    {
+      0, 0, 0, 0,
+      "Precedence (default: column operations first)",
+      UI_GROUP_PRECEDENCE
+    },
+    {
+      "rowfirst",
+      UI_KEY_ROWFIRST,
+      0,
+      0,
+      "Apply row-based operations first.",
+      UI_GROUP_PRECEDENCE,
+      &p->rowfirst,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+
+
+
+
+
+    /* Output columns. */
+    {
+      0, 0, 0, 0,
+      "Columns in output:",
+      UI_GROUP_OUTCOLS
+    },
+    {
+      "catcolumnrawname",
+      UI_KEY_CATCOLUMNRAWNAME,
+      0,
+      0,
+      "Don't touch column names of --catcolumnfile.",
+      UI_GROUP_OUTCOLS,
+      &p->catcolumnrawname,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "colmetadata",
+      UI_KEY_COLMETADATA,
+      "ID,S,S,S",
+      0,
+      "Column metadata (S=STR: Name, Unit, Comments).",
+      UI_GROUP_OUTCOLS,
+      &p->colmetadata,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      gal_options_parse_name_and_strings
+    },
+    {
+      "tovector",
+      UI_KEY_TOVECTOR,
+      "STR,STR[,STR]",
+      0,
+      "Merge column(s) into a vector column.",
+      UI_GROUP_OUTCOLS,
+      &p->tovector,
+      GAL_TYPE_STRLL,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "fromvector",
+      UI_KEY_FROMVECTOR,
+      "STR,INT[,INT]",
+      0,
+      "Extract column(s) from a vector column.",
+      UI_GROUP_OUTCOLS,
+      &p->fromvector,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      gal_options_parse_name_and_sizets
+    },
+    {
+      "keepvectfin",
+      UI_KEY_KEEPVECTFIN,
+      0,
+      0,
+      "Keep inputs of '--tovector' & '--fromvector'.",
+      UI_GROUP_OUTCOLS,
+      &p->keepvectfin,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "transpose",
+      UI_KEY_TRANSPOSE,
+      0,
+      0,
+      "Transpose table (must only contain vector cols).",
+      UI_GROUP_OUTCOLS,
+      &p->transpose,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+
+
+
+
     /* Output Rows */
     {
       0, 0, 0, 0,
