@@ -5,7 +5,7 @@ Table is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2019-2022 Free Software Foundation, Inc.
+Copyright (C) 2019-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -30,11 +30,15 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include <gnuastro/wcs.h>
 #include <gnuastro/type.h>
+#include <gnuastro/pointer.h>
+#include <gnuastro/statistics.h>
 
 #include <gnuastro-internal/checkset.h>
 #include <gnuastro-internal/arithmetic-set.h>
 
 #include "main.h"
+
+#include "ui.h"
 #include "arithmetic.h"
 
 
@@ -58,10 +62,13 @@ arithmetic_add_new_to_end(struct arithmetic_token **list)
 
   /* Initialize its elements. */
   node->next=NULL;
+  node->loadcol=NULL;
   node->name_def=NULL;
   node->name_use=NULL;
   node->constant=NULL;
+  node->id_at_usage=NULL;
   node->index=GAL_BLANK_SIZE_T;
+  node->num_at_usage=GAL_BLANK_SIZE_T;
   node->operator=GAL_ARITHMETIC_OP_INVALID;
 
   /* If the list already has elements, go to the last node in the list and
@@ -135,6 +142,7 @@ arithmetic_operator_name(int operator)
       case ARITHMETIC_TABLE_OP_DISTANCEFLAT: out="distance-flat"; break;
       case ARITHMETIC_TABLE_OP_DATETOMILLISEC: out="date-to-millisec"; break;
       case ARITHMETIC_TABLE_OP_DISTANCEONSPHERE: out="distance-on-sphere"; break;
+      case ARITHMETIC_TABLE_OP_SORTEDTOINTERVAL: out="sorted-to-interval"; break;
       default:
         error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
               "the problem. %d is not a recognized operator code", __func__,
@@ -175,9 +183,13 @@ arithmetic_init_wcs(struct tableparams *p, char *operator)
 /* Set the operator code from the given string. */
 static int
 arithmetic_set_operator(struct tableparams *p, char *string,
-                        size_t *num_operands)
+                        size_t *num_operands, int *inlib)
 {
-  int op=gal_arithmetic_set_operator(string, num_operands);;
+  /* Use the library's main function for its own operators. */
+  int op=gal_arithmetic_set_operator(string, num_operands);
+
+  /* Mark if this operator is in the library or not. */
+  *inlib = op==GAL_ARITHMETIC_OP_INVALID ? 0 : 1;
 
   /* Set the operator and number of operands. */
   if( op==GAL_ARITHMETIC_OP_INVALID )
@@ -195,6 +207,8 @@ arithmetic_set_operator(struct tableparams *p, char *string,
         { op=ARITHMETIC_TABLE_OP_DISTANCEFLAT; *num_operands=0; }
       else if( !strcmp(string, "distance-on-sphere"))
         { op=ARITHMETIC_TABLE_OP_DISTANCEONSPHERE; *num_operands=0; }
+      else if( !strcmp(string, "sorted-to-interval"))
+        { op=ARITHMETIC_TABLE_OP_SORTEDTOINTERVAL; *num_operands=0; }
       else
         { op=GAL_ARITHMETIC_OP_INVALID; *num_operands=GAL_BLANK_INT; }
     }
@@ -216,28 +230,87 @@ arithmetic_set_operator(struct tableparams *p, char *string,
 
 
 
+/* When a column identifier (name or number) is identified, we need to make
+   sure if that the identifier belongs to the first input or not. */
+static void
+arithmetic_init_addcol(char *token, struct arithmetic_token *node,
+                       gal_list_str_t **colstoread, size_t *totcalled,
+                       gal_data_t *colinfo, size_t numcols)
+{
+  char *id;
+  int inmaininput=0;
+  size_t i, num=GAL_BLANK_SIZE_T;
+
+  /* This needs to be defined after 'num'. */
+  void *numptr=&num;
+
+  /* Set the identifier. */
+  id = ( (token[0]=='$' && isdigit(token[1]))
+         ? &token[1]       /* Column num. (starting with '$'). */
+         : token );        /* Column name, just add it.        */
+
+  /* See if the column identifier is a number, then 'num' will not be a
+     blank value. */
+  if( gal_type_from_string(&numptr, id, GAL_TYPE_SIZE_T) )
+    num=GAL_BLANK_SIZE_T;
+
+  /* See if the column exists in the main input catalog. */
+  if(num==GAL_BLANK_SIZE_T)     /* ID is a string. */
+    {
+      for(i=0;i<numcols;++i)
+        if( colinfo[i].name && !strcasecmp(id, colinfo[i].name) )
+          { inmaininput=1; break; }
+    }
+  else                          /* ID is a number. */
+    if(num<=numcols) inmaininput=1;
+
+  /* If the given ID is in the main input, then add it to the list of
+     columns to read ('colstoread') from the main input. Otherwise, keep
+     the ID for usage at the time of arithmetic (can happen when
+     '--catcolumnfile' is used).*/
+  if(inmaininput)
+    {
+      gal_list_str_add(colstoread, id, 1);
+      node->index=*totcalled;
+      *totcalled+=1;
+    }
+  else
+    {
+      gal_checkset_allocate_copy(id, &node->id_at_usage);/* Future usage. */
+      node->num_at_usage=num;   /* Avoid converting to a number again! */
+      node->index=GAL_BLANK_SIZE_T; /* Crash if not used properly! */
+    }
+}
+
+
+
+
+
 /* Initialize each column from an arithmetic operation. */
 void
 arithmetic_init(struct tableparams *p, struct arithmetic_token **arith,
-                gal_list_str_t **toread, size_t *totcalled, char *expression)
+                gal_list_str_t **colstoread, size_t *totcalled,
+                char *expression, gal_data_t *colinfo, size_t numcols)
 {
   void *num;
   size_t one=1;
   uint8_t ntype;
-  char *str, *delimiter=" \t";
+  gal_data_t *col;
   struct arithmetic_token *atmp, *node=NULL;
-  char *token=NULL, *lasttoken=NULL, *saveptr;
+  struct gal_options_common_params *cp=&p->cp;
+  char *token=NULL, *saveptr, *delimiter=" \t";
 
   /* Parse all the given tokens. */
   token=strtok_r(expression, delimiter, &saveptr);
   while(token!=NULL)
     {
       /* Allocate and initialize this arithmetic token. */
-      lasttoken=token;
       node=arithmetic_add_new_to_end(arith);
 
       /* See if the token is an operator, if not check other cases.  */
-      node->operator=arithmetic_set_operator(p, token, &node->num_operands);
+      node->operator=arithmetic_set_operator(p, token,
+                                             &node->num_operands,
+                                             &node->inlib);
       if(node->operator==GAL_ARITHMETIC_OP_INVALID)
         {
           /* Token is a single number.*/
@@ -254,6 +327,13 @@ arithmetic_init(struct tableparams *p, struct arithmetic_token **arith,
               gal_checkset_allocate_copy(token, &node->name_def);
             }
 
+          /* This is a 'load-col' operator. */
+          else if( (col=gal_arithmetic_load_col(token, cp->searchin,
+                                                cp->ignorecase,
+                                                cp->minmapsize,
+                                                cp->quietmmap))!=NULL )
+            node->loadcol=col;
+
           /* Non-pre-defined situation.*/
           else
             {
@@ -268,25 +348,14 @@ arithmetic_init(struct tableparams *p, struct arithmetic_token **arith,
               /* If it wasn't found to be a pre-defined name, then its a
                  column operand (column number or name). */
               if(node->name_use==NULL)
-                {
-                  str = ( (token[0]=='$' && isdigit(token[1]))
-                          ? &token[1]   /* Column number (starting with '$'). */
-                          : token );    /* Column name, just add it.          */
-                  gal_list_str_add(toread, str, 1);
-                  node->index=*totcalled;
-                  *totcalled+=1;
-                }
+                arithmetic_init_addcol(token, node, colstoread,
+                                       totcalled, colinfo, numcols);
             }
         }
 
       /* Go to the next token. */
       token=strtok_r(NULL, delimiter, &saveptr);
     }
-
-  /* A small sanity check: the last added token must be an operator. */
-  if( node==NULL || node->operator==GAL_ARITHMETIC_OP_INVALID )
-    error(EXIT_FAILURE, 0, "last token in arithmetic column ('%s') is not a "
-          "recognized operator", lasttoken);
 }
 
 
@@ -296,7 +365,7 @@ arithmetic_init(struct tableparams *p, struct arithmetic_token **arith,
 /* Set the final index of each package of columns (possibly containing
    processing columns that will change in number and contents).  */
 void
-arithmetic_indexs_final(struct tableparams *p, size_t *colmatch)
+arithmetic_indexs_final(struct tableparams *p)
 {
   size_t startind;
   size_t i, numcols;
@@ -308,7 +377,7 @@ arithmetic_indexs_final(struct tableparams *p, size_t *colmatch)
   p->colarray=gal_list_data_to_array_ptr(p->table, &p->numcolarray);
 
   /* go over each package of columns. */
-  for(tmp=p->outcols;tmp!=NULL;tmp=tmp->next)
+  for(tmp=p->colpack;tmp!=NULL;tmp=tmp->next)
     {
       /* If we are on an arithmetic operation. */
       if(tmp->arith)
@@ -317,33 +386,36 @@ arithmetic_indexs_final(struct tableparams *p, size_t *colmatch)
             if(atmp->index!=GAL_BLANK_SIZE_T)
               {
                 /* Small sanity check. */
-                if(colmatch[atmp->index]!=1)
-                  error(EXIT_FAILURE, 0, "arithmetic operands can (currently) "
-                        "only correspond to a single column");
+                if(p->colmatch[atmp->index]!=1)
+                  error(EXIT_FAILURE, 0, "arithmetic operands can "
+                        "(currently) only correspond to a single "
+                        "column, but more than one match was found "
+                        "for a column you used in column arithmetic");
 
                 /* Update the index in the full list of read columns. */
-                numcols=0; for(i=0;i<atmp->index;++i) numcols+=colmatch[i];
+                numcols=0;
+                for(i=0;i<atmp->index;++i) numcols+=p->colmatch[i];
                 atmp->index=numcols;
               }
         }
+
       /* A simple column. */
       else
         {
           /* See where the starting column for this patch of simple columns
              is. */
           startind=0;
-          for(i=0;i<tmp->start;++i) startind+=colmatch[i];
+          for(i=0;i<tmp->start;++i) startind+=p->colmatch[i];
 
           /* How many of the read columns are associated with the this
              patch of columns. */
           numcols=0;
-          for(i=0;i<tmp->numsimple;++i) numcols+=colmatch[tmp->start+i];
+          for(i=0;i<tmp->numsimple;++i) numcols+=p->colmatch[tmp->start+i];
 
           /* Update the values. */
           tmp->start=startind;
           tmp->numsimple=numcols;
         }
-
     }
 }
 
@@ -369,20 +441,64 @@ arithmetic_indexs_final(struct tableparams *p, size_t *colmatch)
 /*********************************************************************/
 /********************       Low-level tools      *********************/
 /*********************************************************************/
+static void
+arithmetic_placeholder_name(gal_data_t *col)
+{
+  static size_t counter=0;
+
+  /* Increment counter next time this function is called. */
+  ++counter;
+
+  /* Free any possibly existing metadata. */
+  if(col->name)    free(col->name);
+  if(col->unit)    free(col->unit);
+  if(col->comment) free(col->comment);
+
+  /* Set the new meta-data. */
+  errno=0;
+  if( asprintf(&col->name, "ARITH_%zu", counter)==-1 )
+    error(EXIT_FAILURE, errno, "%s: asprintf error for name", __func__);
+  if( asprintf(&col->unit, "arith_unit_%zu", counter)==-1)
+    error(EXIT_FAILURE, errno, "%s: asprintf error for unit", __func__);
+  if( asprintf(&col->comment, "Column from arithmetic operation %zu",
+               counter)==-1 )
+    error(EXIT_FAILURE, errno, "%s: asprintf error for comment", __func__);
+}
+
+
+
+
+
 static gal_data_t *
 arithmetic_stack_pop(gal_data_t **stack, int operator, char *errormsg)
 {
-  gal_data_t *out=*stack;
+  size_t i;
+  gal_data_t *out, *top=*stack;
 
-  /* Update the stack. */
-  if(*stack)
-    *stack=(*stack)->next;
-  else
-    error(EXIT_FAILURE, 0, "not enough operands for '%s'%s",
-          arithmetic_operator_name(operator), errormsg?errormsg:"");
+  /* Update the stack (where necessary). */
+  switch(operator)
+    {
+    /* Operators don't need to pop the dataset from the stack (they just
+       need the metadata). */
+    case GAL_ARITHMETIC_OP_INDEX:
+    case GAL_ARITHMETIC_OP_COUNTER:
+      out=gal_data_alloc_empty(top->ndim, top->minmapsize, top->quietmmap);
+      for(i=0;i<top->ndim;++i) out->dsize[i]=top->dsize[i];
+      out->size=top->size;
+      break;
+
+    /* Operators that actually need the data. */
+    default:
+      out=*stack;
+      if(*stack)
+        *stack=(*stack)->next;
+      else
+        error(EXIT_FAILURE, 0, "not enough operands for '%s'%s",
+              arithmetic_operator_name(operator), errormsg?errormsg:"");
+    }
 
   /* Arithmetic changes the contents of a dataset, so the old name (in the
-     FITS 'EXTNAME' keyword) must not be used beyond this
+     FITS 'EXTNAME' keyword) or units must not be used beyond this
      point. Furthermore, in Arithmetic, the 'name' element is used to
      identify variables (with the 'set-' operator). */
   if(out->name)    { free(out->name);    out->name=NULL;    }
@@ -724,6 +840,75 @@ arithmetic_datetosec(struct tableparams *p, gal_data_t **stack,
 
 
 
+/* Convert the ISO date format to seconds since Unix time. */
+static void
+arithmetic_sortedtointerval(struct tableparams *p, gal_data_t **stack,
+                            int operator)
+{
+  int f32out;
+  size_t i, size;
+  double *in, *min, *max;
+  gal_data_t *min_d, *max_d;
+
+  /* Input dataset. */
+  gal_data_t *in_d=arithmetic_stack_pop(stack, operator, NULL);
+
+  /* Make sure the input is sorted and is numeric */
+  if(in_d->type==GAL_TYPE_STRING)
+    error(EXIT_FAILURE, 0, "the operand given to 'sortedtointerval' "
+          "should have a numeric data type");
+  if(gal_statistics_is_sorted(in_d, 1)==0)
+    error(EXIT_FAILURE, 0, "the operand given to 'sortedtointerval' "
+          "should be sorted");
+
+  /* Convert the input to double precision for internal processing. In the
+     end we'll convert the outputs to float32 unless the input was
+     originally float64. */
+  f32out = in_d->type!=GAL_TYPE_FLOAT64;
+  in_d=gal_data_copy_to_new_type_free(in_d, GAL_TYPE_FLOAT64);
+
+  /* Allocate the output columns (minimum and maximum of each interval). */
+  min_d=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, in_d->dsize, NULL, 0,
+                       in_d->minmapsize, in_d->quietmmap,
+                       NULL, NULL, NULL);
+  max_d=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, in_d->dsize, NULL, 0,
+                       in_d->minmapsize, in_d->quietmmap,
+                       NULL, NULL, NULL);
+
+  /* Fill the minimum and maximum intervals. Just note that the minimum
+     value of the first interval and maximum of the last element need
+     special attention (we'll extrapolate from the after/before elements to
+     preserve constant spacing in case that is the case). */
+  in=in_d->array;
+  size=in_d->size;
+  min=min_d->array;
+  max=max_d->array;
+  min[0]      = in[0]      - (in[1]      - in[0]     )/2;
+  max[0]      = in[0]      + (in[1]      - in[0]     )/2;
+  min[size-1] = in[size-1] - (in[size-1] - in[size-2])/2;
+  max[size-1] = in[size-1] + (in[size-1] - in[size-2])/2;
+  for(i=1;i<size-1;++i)
+    {
+      min[i] = max[i-1];
+      max[i] = in[i] + (in[i+1]-in[i])/2;
+    }
+
+  /* Convert the output to float32, unless the input was float64. */
+  if(f32out)
+    {
+      min_d=gal_data_copy_to_new_type_free(min_d, GAL_TYPE_FLOAT32);
+      max_d=gal_data_copy_to_new_type_free(max_d, GAL_TYPE_FLOAT32);
+    }
+
+  /* Clean up and put the resulting calculation back on the stack. */
+  max_d->next=min_d;
+  gal_data_free(in_d);
+  gal_list_data_add(stack, max_d);
+}
+
+
+
+
 
 
 
@@ -742,34 +927,6 @@ arithmetic_datetosec(struct tableparams *p, gal_data_t **stack,
 /*********************************************************************/
 /********************          Operations        *********************/
 /*********************************************************************/
-static void
-arithmetic_placeholder_name(gal_data_t *col)
-{
-  static size_t counter=0;
-
-  /* Increment counter next time this function is called. */
-  ++counter;
-
-  /* Free any possibly existing metadata. */
-  if(col->name)    free(col->name);
-  if(col->unit)    free(col->unit);
-  if(col->comment) free(col->comment);
-
-  /* Set the new meta-data. */
-  errno=0;
-  if( asprintf(&col->name, "ARITH_%zu", counter)==-1 )
-    error(EXIT_FAILURE, errno, "%s: asprintf error for name", __func__);
-  if( asprintf(&col->unit, "arith_unit_%zu", counter)==-1)
-    error(EXIT_FAILURE, errno, "%s: asprintf error for unit", __func__);
-  if( asprintf(&col->comment, "Column from arithmetic operation %zu",
-               counter)==-1 )
-    error(EXIT_FAILURE, errno, "%s: asprintf error for comment", __func__);
-}
-
-
-
-
-
 static void
 arithmetic_operator_run(struct tableparams *p,
                         struct arithmetic_token *token,
@@ -777,14 +934,14 @@ arithmetic_operator_run(struct tableparams *p,
                         gal_data_t **stack)
 {
   int flags=GAL_ARITHMETIC_FLAGS_BASIC;
-  gal_data_t *d1=NULL, *d2=NULL, *d3=NULL;
+  gal_data_t *d1=NULL, *d2=NULL, *d3=NULL, *d4=NULL;
 
   /* Set the operating-mode flags if necessary. */
   if(p->cp.quiet) flags |= GAL_ARITHMETIC_FLAG_QUIET;
   if(p->envseed)  flags |= GAL_ARITHMETIC_FLAG_ENVSEED;
 
-  /* When 'num_operands!=0', the operator is in the library. */
-  if(token->num_operands)
+  /* If this operator is in the library, we should pop everything here. */
+  if(token->inlib)
     {
       /* Pop the necessary number of operators. Note that the
          operators are poped from a linked list (which is
@@ -793,6 +950,9 @@ arithmetic_operator_run(struct tableparams *p,
          last (right most, in in-fix notation) input operand.*/
       switch(token->num_operands)
         {
+        case 0:
+          break;
+
         case 1:
           d1=arithmetic_stack_pop(stack, token->operator, NULL);
           break;
@@ -808,6 +968,13 @@ arithmetic_operator_run(struct tableparams *p,
           d1=arithmetic_stack_pop(stack, token->operator, NULL);
           break;
 
+        case 4:
+          d4=arithmetic_stack_pop(stack, token->operator, NULL);
+          d3=arithmetic_stack_pop(stack, token->operator, NULL);
+          d2=arithmetic_stack_pop(stack, token->operator, NULL);
+          d1=arithmetic_stack_pop(stack, token->operator, NULL);
+          break;
+
         case -1:
           error(EXIT_FAILURE, 0, "operators with a variable number of "
                 "operands are not yet implemented. Please contact us at "
@@ -815,9 +982,9 @@ arithmetic_operator_run(struct tableparams *p,
           break;
 
         default:
-          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
-                "the problem. '%zu' is not recognized as an operand "
-                "counter (with '%s')", __func__, PACKAGE_BUGREPORT,
+          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s "
+                "to fix the problem. '%zu' is not recognized as an "
+                "operand counter (with '%s')", __func__, PACKAGE_BUGREPORT,
                 token->num_operands,
                 arithmetic_operator_name(token->operator));
         }
@@ -829,7 +996,7 @@ arithmetic_operator_run(struct tableparams *p,
          ignored. */
       gal_list_data_add(stack, gal_arithmetic(token->operator,
                                               p->cp.numthreads,
-                                              flags, d1, d2, d3) );
+                                              flags, d1, d2, d3, d4) );
 
       /* Reset the meta-data for the element that was just put on the
          stack. */
@@ -856,6 +1023,10 @@ arithmetic_operator_run(struct tableparams *p,
           arithmetic_distance(p, stack, token->operator);
           break;
 
+        case ARITHMETIC_TABLE_OP_SORTEDTOINTERVAL:
+          arithmetic_sortedtointerval(p, stack, token->operator);
+          break;
+
         case ARITHMETIC_TABLE_OP_SET:
           gal_arithmetic_set_name(setprm, token->name_def);
           break;
@@ -872,9 +1043,70 @@ arithmetic_operator_run(struct tableparams *p,
 
 
 
+/* When the column identifier (name or number) wasn't in the main input
+   table (for example, it was added with '--catcolumnfile'), we need this
+   function to find the column to work with. */
+gal_data_t *
+arithmetic_read_at_usage(struct tableparams *p,
+                         struct arithmetic_token *token)
+{
+  char *c;
+  size_t i;
+  int hasnewline=0;
+
+  /* If the number is blank, then we have a name and should return the
+     first column in the table that has the given name. */
+  if(token->num_at_usage==GAL_BLANK_SIZE_T)
+    {
+      /* Search all the existing columns in the table. */
+      for(i=0; i<p->numcolarray; ++i)
+        if( p->colarray[i]->name
+            && strcasecmp(p->colarray[i]->name, token->id_at_usage)==0 )
+          return p->colarray[i];
+
+      /* If control reaches here, then the requested name didn't exist in
+         the table. This may happen because the user broke an arithmetic
+         command into muliple lines and forgot to put a back-slash. If so,
+         the string will have a new-line within it.*/
+      for(c=token->id_at_usage;*c!='\0';++c)
+        if(*c=='\n') hasnewline=1;
+
+      /* print the error message. */
+      error(EXIT_FAILURE, 0, "'%s' doesn't correspond to the name of "
+            "any column in the table until this column arithmetic%s",
+            token->id_at_usage, hasnewline?". Because there is a "
+            "new-line in the name, it is highly probable that it "
+            "wasn't actually a name. This can happen when you break "
+            "an arithmetic command into multiple lines, but have "
+            "forgot to put a back-slash ('\\') at the end of a "
+            "broken line":"");
+    }
+
+  /* We already have the desired column number. */
+  else
+    if(token->num_at_usage > p->numcolarray)
+      error(EXIT_FAILURE, 0, "%zu is an invalid column number: the "
+            "table only has %zu columns before this column arithmetic "
+            "step", token->num_at_usage, p->numcolarray);
+    else
+      return p->colarray[token->num_at_usage-1];
+
+  /* If control reaches here, there is a bug! In then end, simply return
+     NULL to avoid compiler warnings. */
+  error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to find the "
+        "cause and fix it. This function should not reach this point",
+        __func__, PACKAGE_BUGREPORT);
+  return NULL;
+}
+
+
+
+
+
 /* Apply reverse polish mechanism for this column. */
 static void
-arithmetic_reverse_polish(struct tableparams *p, struct column_pack *outpack)
+arithmetic_reverse_polish(struct tableparams *p,
+                          struct column_pack *outpack)
 {
   struct arithmetic_token *token;
   gal_data_t *single, *stack=NULL;
@@ -893,22 +1125,44 @@ arithmetic_reverse_polish(struct tableparams *p, struct column_pack *outpack)
       if(token->operator!=GAL_ARITHMETIC_OP_INVALID)
         arithmetic_operator_run(p, token, &setprm, &stack);
 
+
       /* We are on a named variable. */
       else if( token->name_use )
         gal_list_data_add(&stack,
                           gal_arithmetic_set_copy_named(&setprm,
                                                         token->name_use));
 
-      /* Constant number: just put it ontop of the stack. */
+      /* We are on a column loaded from another file. */
+      else if( token->loadcol )
+        {
+          gal_list_data_add(&stack, token->loadcol);
+          token->loadcol=NULL;
+        }
+
+      /* Constant number: just put it on top of the stack. */
       else if(token->constant)
         {
           gal_list_data_add(&stack, token->constant);
           token->constant=NULL;
         }
 
+      /* The column wasn't in the main input. */
+      else if(token->id_at_usage)
+        {
+          gal_list_data_add(&stack, arithmetic_read_at_usage(p, token));
+          token->num_at_usage=GAL_BLANK_SIZE_T;
+          free(token->id_at_usage);
+          token->id_at_usage=NULL;
+        }
+
       /* A column from the table. */
       else if(token->index!=GAL_BLANK_SIZE_T)
-        gal_list_data_add(&stack, p->colarray[token->index]);
+        {
+          if(p->colarray[token->index]->ndim!=1)
+            error(EXIT_FAILURE, 0, "column arithmetic currently only works "
+                  "on single-valued columns, not vector columns");
+          gal_list_data_add(&stack, p->colarray[token->index]);
+        }
 
       /* Un-recognized situation. */
       else
@@ -920,10 +1174,10 @@ arithmetic_reverse_polish(struct tableparams *p, struct column_pack *outpack)
       ++setprm.tokencounter;
     }
 
-  /* Put everything that remains in the stack (reversed) into the final
-     table. Just note that 'gal_list_data_add' behaves differently for
-     lists, so we'll add have to manually set the 'next' element to NULL
-     before adding the column to the final table. */
+ /* Put everything that remains in the stack (reversed) into the final
+    table. Just note that 'gal_list_data_add' behaves differently for
+    lists, so we'll add have to manually set the 'next' element to NULL
+    before adding the column to the final table. */
   gal_list_data_reverse(&stack);
   while(stack!=NULL)
     {
@@ -975,6 +1229,9 @@ arithmetic_operate(struct tableparams *p)
   size_t i;
   struct column_pack *outpack;
 
+  /* Set the final indexs and define 'colarray'. */
+  arithmetic_indexs_final(p);
+
   /* From now on, we will be looking for columns from the index in
      'colarray', so to keep things clean, we'll set all the 'next' elements
      to NULL. */
@@ -985,7 +1242,7 @@ arithmetic_operate(struct tableparams *p)
   p->table=NULL;
 
   /* Go over each package of columns. */
-  for(outpack=p->outcols; outpack!=NULL; outpack=outpack->next)
+  for(outpack=p->colpack; outpack!=NULL; outpack=outpack->next)
     {
       if(outpack->arith)
         arithmetic_reverse_polish(p, outpack);
@@ -1000,4 +1257,10 @@ arithmetic_operate(struct tableparams *p)
      column contents have either been moved into the new table, or have
      already been freed. */
   gal_list_data_reverse(&p->table);
+
+  /* Clean up. */
+  ui_colpack_free(p->colpack);
+  if(p->colarray) free(p->colarray);
+  p->colarray=NULL;
+  p->colpack=NULL;
 }
