@@ -5,7 +5,8 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+     Pedram Ashofteh Ardakani <pedramardakani@pm.me>
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -93,6 +94,8 @@ gal_checkset_allocate_copy_set(char *arg, char **copy, int *set);
 char *
 gal_checkset_dataset_name(char *filename, char *hdu);
 
+char *
+gal_checkset_timestamp(char *filename, char *newext);
 
 
 
@@ -121,10 +124,12 @@ int
 gal_checkset_writable_notexist(char *filename);
 
 void    /* keep==0 && dontdelete==0: file will be deleted if exists.*/
-gal_checkset_writable_remove(char *filename, int keep, int dontdelete);
+gal_checkset_writable_remove(char *filename, char *basename, int keep,
+                             int dontdelete);
 
 int
-gal_checkset_dir_0_file_1(char *name, int dontdelete);
+gal_checkset_dir_0_file_1(struct gal_options_common_params *cp, char *name,
+                          char *basename);
 
 char *
 gal_checkset_automatic_output(struct gal_options_common_params *cp,
