@@ -4,7 +4,7 @@ Function to parse options and configuration file values.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2022 Free Software Foundation, Inc.
+Copyright (C) 2017-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -299,11 +299,20 @@ gal_options_read_interpmetric(struct argp_option *option, char *arg,
 
 gal_data_t *
 gal_options_parse_list_of_numbers(char *string, char *filename,
-                                  size_t lineno);
+                                  size_t lineno, uint8_t type);
+
+gal_list_str_t *
+gal_options_parse_csv_strings_to_list(char *string, char *filename,
+                                      size_t lineno);
 
 gal_data_t *
-gal_options_parse_csv_strings_raw(char *string, char *filename,
-                                  size_t lineno);
+gal_options_parse_csv_strings_to_data(char *string, char *filename,
+                                      size_t lineno);
+
+void *
+gal_options_parse_csv_strings_append(struct argp_option *option, char *arg,
+                                     char *filename, size_t lineno,
+                                     void *junk);
 
 void *
 gal_options_parse_csv_strings(struct argp_option *option, char *arg,
@@ -314,7 +323,8 @@ gal_options_merge_list_of_csv(gal_list_str_t **list);
 
 void *
 gal_options_parse_sizes_reverse(struct argp_option *option, char *arg,
-                                char *filename, size_t lineno, void *params);
+                                char *filename, size_t lineno,
+                                void *params);
 
 void *
 gal_options_parse_csv_float64(struct argp_option *option, char *arg,
@@ -326,11 +336,18 @@ gal_options_read_sigma_clip(struct argp_option *option, char *arg,
 
 void *
 gal_options_parse_name_and_strings(struct argp_option *option, char *arg,
-                                   char *filename, size_t lineno, void *junk);
+                                   char *filename, size_t lineno,
+                                   void *junk);
 
 void *
 gal_options_parse_name_and_float64s(struct argp_option *option, char *arg,
-                                    char *filename, size_t lineno, void *junk);
+                                    char *filename, size_t lineno,
+                                    void *junk);
+
+void *
+gal_options_parse_name_and_sizets(struct argp_option *option, char *arg,
+                                  char *filename, size_t lineno,
+                                  void *junk);
 
 gal_data_t *
 gal_options_parse_colon_sep_csv_raw(char *instring, char *filename,
