@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2022 Free Software Foundation, Inc.
+Copyright (C) 2017-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -128,6 +128,9 @@ gal_blank_number(gal_data_t *input, int updateflag);
 gal_data_t *
 gal_blank_flag(gal_data_t *data);
 
+gal_data_t *
+gal_blank_flag_not(gal_data_t *input);
+
 void
 gal_blank_flag_apply(gal_data_t *input, gal_data_t *flag);
 
@@ -138,7 +141,8 @@ void
 gal_blank_remove_realloc(gal_data_t *input);
 
 gal_data_t *
-gal_blank_remove_rows(gal_data_t *columns, gal_list_sizet_t *column_indexs);
+gal_blank_remove_rows(gal_data_t *columns, gal_list_sizet_t *column_indexs,
+                      int onlydim0);
 
 __END_C_DECLS    /* From C++ preparations */
 
