@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2021, Free Software Foundation, Inc.
+Copyright (C) 2017-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -61,10 +61,14 @@ gal_permutation_check(size_t *permutation, size_t size);
 void
 gal_permutation_apply(gal_data_t *input, size_t *permutation);
 
+void
+gal_permutation_apply_onlydim0(gal_data_t *input, size_t *permutation);
+
 void
 gal_permutation_apply_inverse(gal_data_t *input, size_t *permutation);
 
-
+void
+gal_permutation_transpose_2d(gal_data_t *input);
 
 
 __END_C_DECLS    /* From C++ preparations */
