@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -197,6 +197,11 @@ gal_wcs_to_cd(struct wcsprm *wcs);
 double
 gal_wcs_angular_distance_deg(double r1, double d1, double r2, double d2);
 
+void
+gal_wcs_box_vertices_from_center(double ra_center, double dec_center,
+                                 double ra_delta,  double dec_delta,
+                                 double *out);
+
 double *
 gal_wcs_pixel_scale(struct wcsprm *wcs);
 
