@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -612,13 +612,13 @@ gal_wcs_write(struct wcsprm *wcs, char *filename,
   fitsfile *fptr;
   long *naxes=NULL;
 
-  /* Small sanity checks */
+  /* Small sanity checks. */
   if(wcs==NULL)
     error(EXIT_FAILURE, 0, "%s: input WCS is NULL", __func__);
   if( gal_fits_file_recognized(filename)==0 )
     error(EXIT_FAILURE, 0, "%s: not a FITS suffix", filename);
 
-  /* Open the file for writing */
+  /* Open the file for writing. */
   fptr=gal_fits_open_to_write(filename);
 
   /* Create the FITS file. */
@@ -960,18 +960,19 @@ wcs_coordsys_insys_pole_in_outsys(int insys, int outsys, double *lng2p1,
 
 
 static void
-wcs_coordsys_ctypes(int coordsys, char **clng, char **clat, char **radesys)
+wcs_coordsys_ctypes(int coordsys, char **clng, char **clat, char **radesys,
+                    double *equinox)
 {
   switch( coordsys)
     {
     case GAL_WCS_COORDSYS_EQB1950:
-      *clng="RA";   *clat="DEC";  *radesys="FK4"; break;
+      *clng="RA";   *clat="DEC";  *radesys="FK4"; *equinox=1950; break;
     case GAL_WCS_COORDSYS_EQJ2000:
-      *clng="RA";   *clat="DEC";  *radesys="FK5"; break;
+      *clng="RA";   *clat="DEC";  *radesys="FK5"; *equinox=2000; break;
     case GAL_WCS_COORDSYS_ECB1950:
-      *clng="ELON"; *clat="ELAT"; *radesys="FK4"; break;
+      *clng="ELON"; *clat="ELAT"; *radesys="FK4"; *equinox=1950; break;
     case GAL_WCS_COORDSYS_ECJ2000:
-      *clng="ELON"; *clat="ELAT"; *radesys="FK5"; break;
+      *clng="ELON"; *clat="ELAT"; *radesys="FK5"; *equinox=2000; break;
     case GAL_WCS_COORDSYS_GALACTIC:
       *clng="GLON"; *clat="GLAT"; *radesys=NULL;  break;
     case GAL_WCS_COORDSYS_SUPERGALACTIC:
@@ -1018,7 +1019,7 @@ gal_wcs_coordsys_convert(struct wcsprm *wcs, int outcoordsys)
                                     &lng2p1, &lat2p1, &lng1p2);
 
   /* Find the necessary CTYPE names of the output. */
-  wcs_coordsys_ctypes(outcoordsys, &clng, &clat, &radesys);
+  wcs_coordsys_ctypes(outcoordsys, &clng, &clat, &radesys, &equinox);
 
   /* Convert the WCS's coordinate system (if 'wcsccs' is available). */
 #if GAL_CONFIG_HAVE_WCSLIB_WCSCCS
@@ -1026,7 +1027,10 @@ gal_wcs_coordsys_convert(struct wcsprm *wcs, int outcoordsys)
   wcsccs(out, lng2p1, lat2p1, lng1p2, clng, clat, radesys, equinox, alt);
 #else
 
-  /* Just to avoid compiler warnings for 'equinox' and 'alt'. */
+  /* Just to avoid compiler warnings for 'equinox' and 'alt' (when WCSLIB
+     doesn't have the 'wcsccs' function): these will never be used: if
+     control comes here the function will abort! So don't worry about this
+     command having any kind of effect on the progrma. */
   if(alt) lng2p1+=equinox;
 
   /* Print error message and abort. */
@@ -1277,7 +1281,7 @@ gal_wcs_distortion_convert(struct wcsprm *inwcs, int outdisptype,
           }
         break;
 
-      /* Input's distortion is not yet supported.. */
+      /* Input's distortion is not yet supported. */
       case GAL_WCS_DISTORTION_TPD:
       case GAL_WCS_DISTORTION_DSS:
       case GAL_WCS_DISTORTION_WAT:
@@ -1347,7 +1351,7 @@ gal_wcs_copy(struct wcsprm *wcs)
 
       /* Initialize the allocated WCS structure. The WCSLIB manual says "On
          the first invokation, and only the first invokation, wcsprm::flag
-         must be set to -1 to initialize memory management"*/
+         must be set to -1 to initialize memory management". */
       out->flag=-1;
       wcsini(1, wcs->naxis, out);
 
@@ -1649,7 +1653,7 @@ gal_wcs_warp_matrix(struct wcsprm *wcs)
     error(EXIT_FAILURE, 0, "%s: currently only PCi_ja and CDi_ja keywords "
           "are recognized", __func__);
 
-  /* Return the result */
+  /* Return the result. */
   return out;
 }
 
@@ -1853,6 +1857,44 @@ gal_wcs_angular_distance_deg(double r1, double d1, double r2, double d2)
 
 
 
+/* Calculate the vertices of a box in the given center. The output should
+   have 8 allocated spaces ready to be written into:
+      out[0]=bottom-left-ra
+      out[1]=bottom-left-dec
+      out[2]=bottom-right-ra
+      out[3]=bottom-right-dec
+      out[4]=top-right-ra
+      out[5]=top-right-dec
+      out[6]=top-left-ra
+      out[7]=top-left-dec
+
+  The 'ra_delta' and 'dec_delta' should be the full length of the box along
+  the respective axis (not half of it!). */
+void
+gal_wcs_box_vertices_from_center(double ra_center, double dec_center,
+                                 double ra_delta,  double dec_delta,
+                                 double *out)
+{
+  double corr;
+
+  /* The bottom vertices, note that the positive right ascension is on
+     the left side. */
+  out[1] = out[3] = dec_center - dec_delta/2;
+  corr=1/(2*cos(out[1]*M_PI/180));
+  out[0] = ra_center + ra_delta*corr;
+  out[2] = ra_center - ra_delta*corr;
+
+  /* The top vertices. */
+  out[5] = out[7] = dec_center + dec_delta/2;
+  corr=1/(2*cos(out[5]*M_PI/180));
+  out[4] = ra_center + ra_delta*corr;
+  out[6] = ra_center - ra_delta*corr;
+}
+
+
+
+
+
 /* Return the pixel scale of the dataset in units of the WCS. */
 double *
 gal_wcs_pixel_scale(struct wcsprm *wcs)
@@ -1904,14 +1946,14 @@ gal_wcs_pixel_scale(struct wcsprm *wcs)
         if(a[i*n+j]!=0.0) /* We aren't concerned with 0 valued elements. */
           {
             /* We have to use absolutes because in cases like RA, the
-               diagonal values in different rows may have different signs*/
+               diagonal values in different rows may have different signs. */
             if(fabs(a[i*n+j])<minrow) minrow=fabs(a[i*n+j]);
             if(fabs(a[i*n+j])>maxrow) maxrow=fabs(a[i*n+j]);
           }
 
       /* Do the check, print warning and make correction. */
       if(maxrow!=minrow
-         && maxrow/minrow>1e5    /* The difference between elements is large */
+         && maxrow/minrow>1e5 /* The difference between elements is large. */
          && maxrow/minrow<GAL_WCS_FLTERROR
          && warning_printed==0)
         {
@@ -2192,7 +2234,7 @@ gal_wcs_coverage(char *filename, char *hdu, size_t *ondim,
      dimension: this is usually the first WCS dimension, but it may happen
      that it isn't. Also, note that the WCS may not be celestial/spherical
      at all (hence why we are only doing this for pre-defined celestial
-     'CTYPE's).*/
+     'CTYPE's). */
   for(i=0;i<ndim;++i)
     if(   strncmp(wcs->ctype[i], "RA-",   3)==0 /* Equatorial */
        || strncmp(wcs->ctype[i], "GLON-", 5)==0 /* Galactic */
@@ -2481,7 +2523,7 @@ gal_wcs_img_to_world(gal_data_t *coords, struct wcsprm *wcs, int inplace)
 
   /* Use WCSLIB's wcsp2s for the conversion. We are ignoring the over-all
      status here, because later we will use the 'stat' array to set all bad
-     coordinates to NaN.*/
+     coordinates to NaN. */
   wcsp2s(wcs, ncoord, nelem, pixcrd, imgcrd, phi, theta, world, stat);
 
 
